# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The interface for a dialog that allows the user to select a font. """

import wx

from traits.api import provides

from pyface.font import Font
from pyface.ui_traits import PyfaceFont
from pyface.i_font_dialog import IFontDialog
from .dialog import Dialog

# The WxPython version in a convenient to compare form.
wx_version = tuple(int(x) for x in wx.__version__.split('.')[:3])


@provides(IFontDialog)
class FontDialog(Dialog):
    """ A dialog for selecting fonts.
    """

    # 'IFontDialog' interface ----------------------------------------------

    #: The font in the dialog.
    font = PyfaceFont()

    # ------------------------------------------------------------------------
    # 'IDialog' interface.
    # ------------------------------------------------------------------------

    def _create_contents(self, parent):
        # In wx this is a canned dialog.
        pass

    # ------------------------------------------------------------------------
    # 'IWindow' interface.
    # ------------------------------------------------------------------------

    def close(self):
        font_data = self.control.GetFontData()
        wx_font = font_data.GetChosenFont()
        self.font = Font.from_toolkit(wx_font)
        super(FontDialog, self).close()

    # ------------------------------------------------------------------------
    # 'IWidget' interface.
    # ------------------------------------------------------------------------

    def _create_control(self, parent):
        wx_font = self.font.to_toolkit()
        data = wx.FontData()
        data.SetInitialFont(wx_font)
        dialog = wx.FontDialog(parent, data)
        return dialog



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import unittest

from ..heading_text import HeadingText
from ..layered_panel import LayeredPanel
from ..toolkit import toolkit_object
from ..window import Window

GuiTestAssistant = toolkit_object("util.gui_test_assistant:GuiTestAssistant")
no_gui_test_assistant = GuiTestAssistant.__name__ == "Unimplemented"


@unittest.skipIf(no_gui_test_assistant, "No GuiTestAssistant")
class TestLayeredPanel(unittest.TestCase, GuiTestAssistant):
    def setUp(self):
        GuiTestAssistant.setUp(self)
        self.window = Window()
        self.window.create()

    def tearDown(self):
        if self.widget.control is not None:
            with self.delete_widget(self.widget.control):
                self.widget.destroy()

        if self.window.control is not None:
            with self.delete_widget(self.window.control):
                self.window.destroy()

        del self.widget
        del self.window
        GuiTestAssistant.tearDown(self)

    def test_lifecycle(self):
        # test that create and destory work
        self.widget = LayeredPanel()

        self.assertIsNone(self.widget.control)

        with self.event_loop():
            self.widget.create(parent=self.window.control)

        self.assertIsNotNone(self.widget.control)

        with self.event_loop():
            self.widget.destroy()

        self.assertIsNone(self.widget.control)

    def test_add_layer(self):
        # test that a layer can be added
        self.widget = LayeredPanel()
        layer_widget = HeadingText()

        with self.event_loop():
            self.widget.create(parent=self.window.control)
            layer_widget.create(parent=self.window.control)

        try:
            with self.event_loop():
                self.widget.add_layer("test 1", layer_widget.control)

            self.assertIn("test 1", self.widget._layers)
            self.assertIs(self.widget._layers["test 1"], layer_widget.control)
        finally:
            with self.event_loop():
                layer_widget.destroy()
                self.widget.destroy()

    def test_show_layer(self):
        # test that a layer can be shown
        self.widget = LayeredPanel()
        layer_widget_1 = HeadingText()
        layer_widget_2 = HeadingText()

        with self.event_loop():
            self.widget.create(parent=self.window.control)
            layer_widget_1.create(parent=self.window.control)
            layer_widget_2.create(parent=self.window.control)
            self.widget.add_layer("test 1", layer_widget_1.control)
            self.widget.add_layer("test 2", layer_widget_2.control)

        try:
            with self.event_loop():
                self.widget.show_layer("test 1")

            self.assertEqual(self.widget.current_layer_name, "test 1")
            self.assertIs(self.widget.current_layer, layer_widget_1.control)

            with self.event_loop():
                self.widget.show_layer("test 2")

            self.assertEqual(self.widget.current_layer_name, "test 2")
            self.assertIs(self.widget.current_layer, layer_widget_2.control)
        finally:
            with self.event_loop():
                layer_widget_1.destroy()
                layer_widget_2.destroy()
                self.widget.destroy()

    def test_has_layer(self):
        # test that a has_layer works
        self.widget = LayeredPanel()
        layer_widget_1 = HeadingText()
        layer_widget_2 = HeadingText()

        with self.event_loop():
            self.widget.create(parent=self.window.control)
            layer_widget_1.create(parent=self.window.control)
            layer_widget_2.create(parent=self.window.control)
            self.widget.add_layer("test 1", layer_widget_1.control)
            self.widget.add_layer("test 2", layer_widget_2.control)

        try:
            self.assertTrue(self.widget.has_layer("test 1"))
            self.assertTrue(self.widget.has_layer("test 2"))
        finally:
            with self.event_loop():
                layer_widget_1.destroy()
                layer_widget_2.destroy()

        with self.event_loop():
            self.widget.destroy()



# Copyright (c) 2014-2023 by Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!

import sys
import unittest

from pyface.qt import QtCore
from pyface.tasks.api import (
    DockPane, EditorAreaPane, PaneItem, Task, TaskFactory, TaskLayout,
    TasksApplication)


class MyDockPane(DockPane):
    id = "my_dock_pane"
    name = u"My dock pane"


class MyTask(Task):
    id = "my_task"
    name = u"My task"

    def _default_layout_default(self):
        return TaskLayout(left=PaneItem("my_dock_pane", width=200))

    def create_central_pane(self):
        return EditorAreaPane()

    def create_dock_panes(self):
        return [MyDockPane()]


class MyApplication(TasksApplication):
    id = "my_application"
    name = u"My application"

    def _task_factories_default(self):
        return [
            TaskFactory(
                id="my_task_factory",
                name=u"My task factory",
                factory=MyTask,
            ),
        ]


class TestDockPane(unittest.TestCase):

    @unittest.skipUnless(sys.platform == "darwin", "only applicable to macOS")
    def test_dock_windows_visible_on_macos(self):
        # Regression test for enthought/pyface#427: check that dock panes
        # are displayed on macOS even when the application doesn't have
        # focus.

        tool_attributes = []

        def check_panes_and_exit(app_event):
            app = app_event.application
            for window in app.windows:
                for dock_pane in window.dock_panes:
                    attr = dock_pane.control.testAttribute(
                        QtCore.Qt.WidgetAttribute.WA_MacAlwaysShowToolWindow)
                    tool_attributes.append(attr)

            app.exit()

        app = MyApplication()
        app.on_trait_change(check_panes_and_exit, "application_initialized")
        app.run()

        self.assertTrue(tool_attributes)
        for attr in tool_attributes:
            self.assertTrue(attr)

    def test_dock_windows_undock(self):
        # Regression test for enthought/pyface#1028: check that undocking
        # dockpanes doesn't crash

        tool_attributes = []

        def check_panes_and_exit(app_event):
            app = app_event.application
            app.windows[0].dock_panes[0].control.setFloating(True)
            for window in app.windows:
                for dock_pane in window.dock_panes:
                    attr = dock_pane.dock_area
                    tool_attributes.append(attr)

            app.exit()

        app = MyApplication()
        app.on_trait_change(check_panes_and_exit, "application_initialized")
        app.run()

        self.assertTrue(tool_attributes)
        for attr in tool_attributes:
            self.assertEqual(attr, 'left')



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import unittest

from ..heading_text import HeadingText
from ..image_resource import ImageResource
from ..toolkit import toolkit_object
from ..window import Window

GuiTestAssistant = toolkit_object("util.gui_test_assistant:GuiTestAssistant")
no_gui_test_assistant = GuiTestAssistant.__name__ == "Unimplemented"

is_wx = toolkit_object.toolkit == "wx"


@unittest.skipIf(no_gui_test_assistant, "No GuiTestAssistant")
class TestHeadingText(unittest.TestCase, GuiTestAssistant):
    def setUp(self):
        GuiTestAssistant.setUp(self)
        self.window = Window()
        self.window.create()

    def tearDown(self):
        if self.widget.control is not None:
            with self.delete_widget(self.window.control):
                self.widget.destroy()

        if self.window.control is not None:
            with self.delete_widget(self.window.control):
                self.window.destroy()

        del self.widget
        del self.window
        GuiTestAssistant.tearDown(self)

    def test_lifecycle(self):
        # test that create/destroy works
        self.widget = HeadingText(self.window.control)

        self.assertIsNone(self.widget.control)

        with self.event_loop():
            self.widget.create(parent=self.window.control)

        self.assertIsNotNone(self.widget.control)

        with self.event_loop():
            self.widget.destroy()

    def test_one_stage_create(self):
        # test that automatic creation works
        with self.event_loop():
            with self.assertWarns(DeprecationWarning):
                self.widget = HeadingText(self.window.control, create=True)

        self.assertIsNotNone(self.widget.control)

        with self.event_loop():
            self.widget.destroy()

    def test_two_stage_create(self):
        # test that create=False works
        with self.assertWarns(DeprecationWarning):
            self.widget = HeadingText(create=False)

        self.assertIsNone(self.widget.control)

        with self.event_loop():
            self.widget.create(parent=self.window.control)

        self.assertIsNotNone(self.widget.control)

        with self.event_loop():
            self.widget.destroy()

    def test_text(self):
        # test that create works with text
        with self.event_loop():
            self.widget = HeadingText(
                self.window.control,
                text="Hello",
            )
            self.widget.create()

        self.assertEqual(self.widget.text, "Hello")
        self.assertEqual(self.widget._get_control_text(), "Hello")

        with self.event_loop():
            self.widget.destroy()

    @unittest.skipUnless(is_wx, "Only Wx supports background images")
    def test_image(self):
        # test that image raises a deprecation warning
        with self.event_loop():
            with self.assertWarns(DeprecationWarning):
                self.widget = HeadingText(
                    self.window.control,
                    image=ImageResource("core.png"),
                )



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""

API for the ``pyface.timer`` subpackage.

- :func:`~.do_later`
- :func:`~.do_after`
- :class:`~.DoLaterTimer`
- :class:`~.CallbackTimer`
- :class:`~.EventTimer`
- :class:`~.Timer`

Interfaces
----------
- :class:`~.ICallbackTimer`
- :class:`~.IEventTimer`
- :class:`~.ITimer`

"""

from .i_timer import ICallbackTimer, IEventTimer, ITimer


# ----------------------------------------------------------------------------
# Deferred imports
# ----------------------------------------------------------------------------

# These imports have the side-effect of performing toolkit selection

# These are pyface.undo.* imports that have selection as a side-effect
# TODO: refactor to delay imports where possible
_relative_imports = {
    'do_later': 'do_later',
    'do_after': 'do_later',
    'DoLaterTimer': 'do_later',
    'CallbackTimer': 'timer',
    'EventTimer': 'timer',
    'Timer': 'timer',
}


def __getattr__(name):
    """Lazily load attributes with side-effects

    In particular, lazily load toolkit backend names.  For efficiency, lazily
    loaded objects are injected into the module namespace
    """
    # sentinel object for no result
    not_found = object()
    result = not_found

    if name in _relative_imports:
        from importlib import import_module
        source = _relative_imports[name]
        module = import_module(f"pyface.timer.{source}")
        result = getattr(module, name)

    if result is not_found:
        raise AttributeError(f"module {__name__!r} has no attribute {name!r}")

    globals()[name] = result
    return result


# ----------------------------------------------------------------------------
# Introspection support
# ----------------------------------------------------------------------------

def __dir__():
    return sorted(set(globals()) | set(_relative_imports))



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

import sys
import unittest
import warnings


class TestPyfaceQtImports(unittest.TestCase):

    def test_imports(self):
        # check that all Qt API imports work
        import pyface.qt.QtCore  # noqa: F401
        import pyface.qt.QtGui  # noqa: F401
        import pyface.qt.QtNetwork  # noqa: F401
        import pyface.qt.QtOpenGL  # noqa: F401
        import pyface.qt.QtOpenGLWidgets  # noqa: F401
        import pyface.qt.QtSvg  # noqa: F401
        import pyface.qt.QtSvgWidgets  # noqa: F401
        import pyface.qt.QtTest  # noqa: F401
        import pyface.qt.QtMultimedia  # noqa: F401
        import pyface.qt.QtMultimediaWidgets  # noqa: F401
        import pyface.qt.QtWidgets  # noqa: F401

    @unittest.skipIf(sys.version_info > (3, 6), "WebKit is not available")
    def test_import_web_kit(self):
        import pyface.qt.QtWebKit  # noqa: F401

    def test_import_QtScript(self):
        # QtScript is not supported on PyQt5/PySide2 and
        # this import will raise a deprecation warning.
        with warnings.catch_warnings(record=True) as w:
            # Cause all warnings to always be triggered.
            warnings.simplefilter("always", category=DeprecationWarning)

            import pyface.qt.QtScript  # noqa: F401

        self.assertTrue(len(w) == 1)
        for warn in w:
            self.assertEqual(warn.category, DeprecationWarning)

    def test_import_qopenglwidget(self):
        # smoke test
        from pyface.qt.QtOpenGL import QOpenGLWidget  # noqa: F401



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""
Font conversion utilities

This module provides facilities for converting between pyface Font objects
and Wx Font objects, trying to keep as much similarity as possible between
them.
"""

import wx

# font weight and size features changed in wxPython 4.1/wxWidgets 3.1
wx_python_4_1 = (wx.VERSION >= (4, 1))


wx_family_to_generic_family = {
    wx.FONTFAMILY_DEFAULT: 'default',
    wx.FONTFAMILY_DECORATIVE: 'fantasy',
    wx.FONTFAMILY_ROMAN: 'serif',
    wx.FONTFAMILY_SCRIPT: 'cursive',
    wx.FONTFAMILY_SWISS: 'sans-serif',
    wx.FONTFAMILY_MODERN: 'monospace',
    wx.FONTFAMILY_TELETYPE: 'typewriter',
}
generic_family_to_wx_family = {
    'default': wx.FONTFAMILY_DEFAULT,
    'fantasy': wx.FONTFAMILY_DECORATIVE,
    'decorative': wx.FONTFAMILY_DECORATIVE,
    'serif': wx.FONTFAMILY_ROMAN,
    'roman': wx.FONTFAMILY_ROMAN,
    'cursive': wx.FONTFAMILY_SCRIPT,
    'script': wx.FONTFAMILY_SCRIPT,
    'sans-serif': wx.FONTFAMILY_SWISS,
    'swiss': wx.FONTFAMILY_SWISS,
    'monospace': wx.FONTFAMILY_MODERN,
    'modern': wx.FONTFAMILY_MODERN,
    'typewriter': wx.FONTFAMILY_TELETYPE,
    'teletype': wx.FONTFAMILY_TELETYPE,
}

if wx_python_4_1:
    weight_to_wx_weight = {
        100: wx.FONTWEIGHT_THIN,
        200: wx.FONTWEIGHT_EXTRALIGHT,
        300: wx.FONTWEIGHT_LIGHT,
        400: wx.FONTWEIGHT_NORMAL,
        500: wx.FONTWEIGHT_MEDIUM,
        600: wx.FONTWEIGHT_SEMIBOLD,
        700: wx.FONTWEIGHT_BOLD,
        800: wx.FONTWEIGHT_EXTRABOLD,
        900: wx.FONTWEIGHT_HEAVY,
        1000: wx.FONTWEIGHT_EXTRAHEAVY,
    }
    wx_weight_to_weight = {
        wx.FONTWEIGHT_THIN: 'thin',
        wx.FONTWEIGHT_EXTRALIGHT: 'extra-light',
        wx.FONTWEIGHT_LIGHT: 'light',
        wx.FONTWEIGHT_NORMAL: 'normal',
        wx.FONTWEIGHT_MEDIUM: 'medium',
        wx.FONTWEIGHT_SEMIBOLD: 'semibold',
        wx.FONTWEIGHT_BOLD: 'bold',
        wx.FONTWEIGHT_EXTRABOLD: 'extra-bold',
        wx.FONTWEIGHT_HEAVY: 'heavy',
        wx.FONTWEIGHT_EXTRAHEAVY: 'extra-heavy',
        wx.FONTWEIGHT_MAX: 'extra-heavy',
    }
else:
    weight_to_wx_weight = {
        100: wx.FONTWEIGHT_LIGHT,
        200: wx.FONTWEIGHT_LIGHT,
        300: wx.FONTWEIGHT_LIGHT,
        400: wx.FONTWEIGHT_NORMAL,
        500: wx.FONTWEIGHT_NORMAL,
        600: wx.FONTWEIGHT_BOLD,
        700: wx.FONTWEIGHT_BOLD,
        800: wx.FONTWEIGHT_BOLD,
        900: wx.FONTWEIGHT_MAX,
        1000: wx.FONTWEIGHT_MAX,
    }
    wx_weight_to_weight = {
        wx.FONTWEIGHT_LIGHT: 'light',
        wx.FONTWEIGHT_NORMAL: 'normal',
        wx.FONTWEIGHT_BOLD: 'bold',
        wx.FONTWEIGHT_MAX: 'extra-heavy',
    }

style_to_wx_style = {
    'normal': wx.FONTSTYLE_NORMAL,
    'oblique': wx.FONTSTYLE_SLANT,
    'italic': wx.FONTSTYLE_ITALIC,
}
wx_style_to_style = {value: key for key, value in style_to_wx_style.items()}


def font_to_toolkit_font(font):
    """ Convert a Pyface font to a wx.font Font.

    Wx fonts have no notion of stretch values or small-caps or overline
    variants, so these are ignored when converting.

    Parameters
    ----------
    font : pyface.font.Font
        The Pyface font to convert.

    Returns
    -------
    wx_font : wx.font.Font
        The best matching wx font.
    """
    size = int(font.size)
    for family in font.family:
        if family in generic_family_to_wx_family:
            default_family = generic_family_to_wx_family[family]
            break
    else:
        default_family = wx.FONTFAMILY_DEFAULT
    weight = weight_to_wx_weight[font.weight_]
    style = style_to_wx_style[font.style]
    underline = ('underline' in font.decorations)

    # get a default font candidate
    wx_font = wx.Font(size, default_family, style, weight, underline)
    for face in font.family:
        # don't try to match generic family
        if face in generic_family_to_wx_family:
            break
        wx_font = wx.Font(
            size, default_family, style, weight, underline, face)
        # we have a match, so stop
        if wx_font.GetFaceName().lower() == face.lower():
            break

    wx_font.SetStrikethrough('strikethrough' in font.decorations)
    return wx_font


def toolkit_font_to_properties(toolkit_font):
    """ Convert a Wx Font to a dictionary of font properties.

    Parameters
    ----------
    toolkit_font : wx.font.Font
        The Wx font to convert.

    Returns
    -------
    properties : dict
        Font properties suitable for use in creating a Pyface Font.
    """
    family = wx_family_to_generic_family[toolkit_font.GetFamily()]
    face = toolkit_font.GetFaceName()
    if wx_python_4_1:
        size = toolkit_font.GetFractionalPointSize()
    else:
        size = toolkit_font.GetPointSize()
    style = wx_style_to_style[toolkit_font.GetStyle()]
    weight = wx_weight_to_weight[toolkit_font.GetWeight()]
    decorations = set()
    if toolkit_font.GetUnderlined():
        decorations.add('underline')
    if toolkit_font.GetStrikethrough():
        decorations.add('strikethrough')

    return {
        'family': [face, family],
        'size': size,
        'weight': weight,
        'stretch': 'normal',
        'style': style,
        'variants': set(),
        'decorations': decorations,
    }



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The 'null' specific implementations of the action manager internal classes.
"""


from traits.api import Any, Bool, HasTraits


class _MenuItem(HasTraits):
    """ A menu item representation of an action item. """

    # '_MenuItem' interface ------------------------------------------------

    # Is the item checked?
    checked = Bool(False)

    # A controller object we delegate taking actions through (if any).
    controller = Any()

    # Is the item enabled?
    enabled = Bool(True)

    # Is the item visible?
    visible = Bool(True)

    # The radio group we are part of (None if the menu item is not part of such
    # a group).
    group = Any()

    # ------------------------------------------------------------------------
    # 'object' interface.
    # ------------------------------------------------------------------------

    def __init__(self, parent, menu, item, controller):
        """ Creates a new menu item for an action item. """

        self.item = item

        self.control_id = 1
        self.control = None

        if controller is not None:
            self.controller = controller
            controller.add_to_menu(self)


class _Tool(HasTraits):
    """ A tool bar tool representation of an action item. """

    # '_Tool' interface ----------------------------------------------------

    # Is the item checked?
    checked = Bool(False)

    # A controller object we delegate taking actions through (if any).
    controller = Any()

    # Is the item enabled?
    enabled = Bool(True)

    # Is the item visible?
    visible = Bool(True)

    # The radio group we are part of (None if the tool is not part of such a
    # group).
    group = Any()

    # ------------------------------------------------------------------------
    # 'object' interface.
    # ------------------------------------------------------------------------

    def __init__(
        self, parent, tool_bar, image_cache, item, controller, show_labels
    ):
        """ Creates a new tool bar tool for an action item. """

        self.item = item
        self.tool_bar = tool_bar

        self.control_id = 1
        self.control = None
        if controller is not None:
            self.controller = controller
            controller.add_to_toolbar(self)


class _PaletteTool(HasTraits):
    """ A tool palette representation of an action item. """

    # '_PaletteTool' interface ---------------------------------------------

    # The radio group we are part of (None if the tool is not part of such a
    # group).
    group = Any()

    # ------------------------------------------------------------------------
    # 'object' interface.
    # ------------------------------------------------------------------------

    def __init__(self, tool_palette, image_cache, item, show_labels):
        """ Creates a new tool palette tool for an action item. """

        self.item = item
        self.tool_palette = tool_palette



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traits.api import Instance, List, Property

from pyface.action.schema.action_manager_builder import ActionManagerBuilder
from pyface.action.schema.schema_addition import SchemaAddition
from pyface.tasks.task import Task


class TaskActionManagerBuilder(ActionManagerBuilder):
    """ ActionManagerBuilder for Tasks.

    This provides some additional convenience methods for extracting schema
    information from a task and using it to build menu bar and toolbar
    managers directly.
    """

    #: The Task to build menubars and toolbars for.
    task = Instance(Task)

    #: The schema additions provided by the Task.
    additions = Property(List(SchemaAddition), observe='task.extra_actions')

    # ------------------------------------------------------------------------
    # 'TaskActionManagerBuilder' interface.
    # ------------------------------------------------------------------------

    def create_menu_bar_manager(self):
        """ Create a menu bar manager from the task's menu bar schema and
            additions.
        """
        if self.task.menu_bar:
            return self.create_action_manager(self.task.menu_bar)
        return None

    def create_tool_bar_managers(self):
        """ Create tool bar managers from the tasks's tool bar schemas and
            additions.
        """
        schemas = self.task.tool_bars + self.get_additional_toolbar_schemas()
        return [
            self.create_action_manager(schema)
            for schema in self._get_ordered_schemas(schemas)
        ]

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    # Trait initializers ---------------------------------------------------

    def _controller_default(self):
        from .task_action_controller import TaskActionController

        return TaskActionController(task=self.task)

    # Trait properties -----------------------------------------------------

    def _get_additions(self):
        # keep synchronized to task's extra actions, since that may change
        if self.task:
            return self.task.extra_actions
        else:
            return []



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Utility functions for handling Qt dates and times. """

from pyface.qt import is_pyqt
from pyface.qt.QtCore import QTime


def qtime_to_pytime(qtime):
    """ Convert a QTime to a Python datetime.time

    This is needed to paper over the differences between PyQt and PySide.

    Parameters
    ----------
    qtime : QTime
        The Qt QTime to convert

    Returns
    -------
    pytime : datetime.time
        The corresponding datetime.time.
    """
    if is_pyqt:
        return qtime.toPyTime()
    else:
        return qtime.toPython()


def pytime_to_qtime(pytime):
    """ Convert a Python datetime.time to a QTime

    This is a convenience function to construct a qtime from a Python time.
    This loses any :attr:`fold` information in the Python time.

    Parameters
    ----------
    pytime : datetime.time
        The datetime.time to convert

    Returns
    -------
    qtime : QTime
        The corresponding Qt QTime.
    """
    return QTime(
        pytime.hour,
        pytime.minute,
        pytime.second,
        pytime.microsecond // 1000
    )



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""
Qt-specific tests for the Window class
"""

import unittest

from pyface.api import Window
from pyface.qt import QtGui
from pyface.ui.qt.util.gui_test_assistant import GuiTestAssistant


class TestWindow(GuiTestAssistant, unittest.TestCase):

    def test_close_last_window(self):
        window = Window()

        last_window_closed = []

        def handle_last_window_closed():
            last_window_closed.append('fired')

        QtGui.QGuiApplication.instance().lastWindowClosed.connect(
            handle_last_window_closed
        )

        try:
            with self.event_loop():
                window.create()

            self.gui.invoke_later(window.control.close)
            self.event_loop_helper.event_loop_with_timeout()
        finally:
            QtGui.QGuiApplication.instance().lastWindowClosed.disconnect(
                handle_last_window_closed
            )

        self.assertEqual(last_window_closed, ['fired'])



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""

API for the ``pyface.wizard`` subpackage.

- :class:`~.WizardPage`
- :class:`~.Wizard`
- :class:`~.WizardController`
- :class:`~.ChainedWizard`
- :class:`~.ChainedWizardController`

Interfaces
----------

- :class:`~.IWizardPage`
- :class:`~.IWizard`
- :class:`~.IWizardController`

"""

from .chained_wizard_controller import ChainedWizardController
from .i_wizard_page import IWizardPage
from .i_wizard import IWizard
from .i_wizard_controller import IWizardController
from .wizard_controller import WizardController


# ----------------------------------------------------------------------------
# Deferred imports
# ----------------------------------------------------------------------------

# These imports have the side-effect of performing toolkit selection

_toolkit_imports = {
    'Wizard': 'wizard',
    'WizardPage': 'wizard_page',
}

# These are pyface.* imports that have selection as a side-effect
_relative_imports = {
    'ChainedWizard': "chained_wizard",
}


def __getattr__(name):
    """Lazily load attributes with side-effects

    In particular, lazily load toolkit backend names.  For efficiency, lazily
    loaded objects are injected into the module namespace
    """
    # sentinel object for no result
    not_found = object()
    result = not_found

    if name in _relative_imports:
        from importlib import import_module
        source = _relative_imports[name]
        module = import_module(f"pyface.wizard.{source}")
        result = getattr(module, name)

    elif name in _toolkit_imports:
        from pyface.toolkit import toolkit_object
        source = _toolkit_imports[name]
        result = toolkit_object(f"wizard.{source}:{name}")

    if result is not_found:
        raise AttributeError(f"module {__name__!r} has no attribute {name!r}")

    globals()[name] = result
    return result


# ----------------------------------------------------------------------------
# Introspection support
# ----------------------------------------------------------------------------

# the list of available names we report for introspection purposes
_extra_names = set(_toolkit_imports) | set(_relative_imports)


def __dir__():
    return sorted(set(globals()) | _extra_names)



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The Qt-specific implementation of the text field class """


from traits.api import Any, provides

from pyface.fields.i_field import IField, MField
from pyface.ui.qt.layout_widget import LayoutWidget
from pyface.ui.qt.util.alignment import (
    alignment_to_qalignment, qalignment_to_alignment
)


@provides(IField)
class Field(MField, LayoutWidget):
    """ The Qt-specific implementation of the field class

    This is an abstract class which is not meant to be instantiated.  Because
    many concrete QWidgets provide a `value` property, the default getters and
    setters target this.
    """

    #: The value held by the field.
    value = Any()

    # ------------------------------------------------------------------------
    # Private interface
    # ------------------------------------------------------------------------

    def _get_control_value(self):
        """ Toolkit specific method to get the control's value. """
        return self.control.value()

    def _set_control_value(self, value):
        """ Toolkit specific method to set the control's value. """
        self.control.setValue(value)

    def _get_control_alignment(self):
        """ Toolkit specific method to get the control's alignment. """
        # default implementation
        return qalignment_to_alignment(self.control.alignment())

    def _set_control_alignment(self, alignment):
        """ Toolkit specific method to set the control's alignment. """
        self.control.setAlignment(alignment_to_qalignment(alignment))



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
# (C) Copyright 2007 Riverbank Computing Limited
# This software is provided without warranty under the terms of the BSD license.
# However, when used with the GPL version of PyQt the additional terms described in the PyQt GPL exception also apply

from traits.api import HasTraits, Int, List, Property, provides, Tuple

from pyface.i_system_metrics import ISystemMetrics, MSystemMetrics
from pyface.qt import QtGui


@provides(ISystemMetrics)
class SystemMetrics(MSystemMetrics, HasTraits):
    """ The toolkit specific implementation of a SystemMetrics.  See the
    ISystemMetrics interface for the API documentation.
    """

    # 'ISystemMetrics' interface -------------------------------------------

    #: The width of the main screen in pixels.
    screen_width = Property(Int)

    #: The height of the main screen in pixels.
    screen_height = Property(Int)

    #: The height and width of each screen in pixels
    screen_sizes = Property(List(Tuple(Int, Int)))

    #: Background color of a standard dialog window as a tuple of RGB values
    #: between 0.0 and 1.0.
    dialog_background_color = Property(Tuple)

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _get_screen_width(self):
        screens = self.screen_sizes
        if len(screens) == 0:
            return 0
        else:
            return screens[0][0]

    def _get_screen_height(self):
        screens = self.screen_sizes
        if len(screens) == 0:
            return 0
        else:
            return screens[0][1]

    def _get_screen_sizes(self):
        screens = QtGui.QApplication.instance().screens()
        return [
            (
                screen.availableGeometry().width(),
                screen.availableGeometry().height(),
            )
            for screen in screens
        ]

    def _get_dialog_background_color(self):
        color = (
            QtGui.QApplication.instance()
            .palette()
            .color(QtGui.QPalette.ColorRole.Window)
        )

        return (color.redF(), color.greenF(), color.blueF())



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import unittest

from traits.testing.optional_dependencies import numpy as np, requires_numpy

from ..image_field import ImageField
from .field_mixin import FieldMixin


@requires_numpy
class ImageFieldMixin(FieldMixin, unittest.TestCase):

    def setUp(self):
        super().setUp()
        self.data = np.full((32, 64, 4), 0xee, dtype='uint8')

    def _create_widget_simple(self, **traits):
        traits.setdefault("tooltip", "Dummy")
        return ImageField(**traits)

    # Tests ------------------------------------------------------------------

    def test_image_field(self):
        self._create_widget_control()

        self.widget.value = 'splash'
        self.gui.process_events()

        self.assertIsNotNone(self.widget._get_control_value())



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


""" Enthought pyface package component
"""


import os
import tempfile
from io import BytesIO


import wx


from pyface.resource.api import ResourceFactory


class PyfaceResourceFactory(ResourceFactory):
    """ The implementation of a shared resource manager. """

    # ------------------------------------------------------------------------
    # 'ResourceFactory' toolkit interface.
    # ------------------------------------------------------------------------

    def image_from_file(self, filename):
        """ Creates an image from the data in the specified filename. """

        # N.B 'wx.BITMAP_TYPE_ANY' tells wxPython to attempt to autodetect the
        # --- image format.
        return wx.Image(filename, wx.BITMAP_TYPE_ANY)

    def image_from_data(self, data, filename=None):
        """ Creates an image from the specified data. """
        return wx.Image(BytesIO(data))

        handle = None
        if filename is None:
            # If there is currently no way in wx to create an image from data,
            # we have write it out to a temporary file and then read it back in:
            handle, filename = tempfile.mkstemp()

        # Write it out...
        tf = open(filename, "wb")
        tf.write(data)
        tf.close()

        # ... and read it back in!  Lovely 8^()
        image = wx.Image(filename, wx.BITMAP_TYPE_ANY)

        # Remove the temporary file.
        if handle is not None:
            os.close(handle)
            os.unlink(filename)

        return image



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traits.api import Enum, Int, Interface, Tuple

#: Value that indicates the default size values should be used.
DEFAULT_SIZE = -1

#: Trait for sizes of widgets.
Size = Tuple(Int(DEFAULT_SIZE), Int(DEFAULT_SIZE))

#: Trait for size policy values.
SizePolicy = Enum("default", "fixed", "preferred", "expand")


class ILayoutItem(Interface):
    """ An item that can participate in layout. """

    #: The minimum size that the item can take.
    minimum_size = Size

    #: The maximum size that the item can take.
    maximum_size = Size

    #: Weight factor used to distribute extra space between widgets.
    stretch = Tuple(Int, Int)

    #: How the item should behave when more space is available.
    size_policy = Tuple(SizePolicy, SizePolicy)



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""

API for the ``pyface.data_view`` subpackage.
Note that this public-facing API is provisional and may change in future
minor releases until Pyface 8.

- :class:`~.AbstractDataExporter`
- :class:`~.AbstractDataModel`
- :class:`~.AbstractValueType`
- :class:`~.DataViewWidget`
- :class:`~.DataWrapper`

Data Formats
------------

- :class:`~.DataFormat`
- :func:`~.text_format`
- :attr:`~.csv_format`
- :attr:`~.csv_column_format`
- :attr:`~.csv_row_format`
- :attr:`~.html_format`
- :attr:`~.npy_format`
- :attr:`~.standard_text_format`
- :attr:`~.table_format`
- :attr:`~.text_column_format`
- :attr:`~.text_row_format`
- :func:`~.from_csv`
- :func:`~.from_csv_column`
- :func:`~.from_csv_row`
- :func:`~.from_json`
- :func:`~.from_npy`
- :func:`~.to_csv`
- :func:`~.to_csv_column`
- :func:`~.to_csv_row`
- :func:`~.to_json`
- :func:`~.to_npy`

Index Managers
--------------

- :class:`~.AbstractIndexManager`
- :class:`~.IntIndexManager`
- :class:`~.TupleIndexManager`

Exceptions
----------
- :class:`~.DataViewError`
- :class:`~.DataViewGetError`
- :class:`~.DataViewSetError`

Interfaces
----------
- :class:`~.IDataViewWidget`
- :class:`~.IDataWrapper`

"""

from pyface.data_view.abstract_data_exporter import AbstractDataExporter
from pyface.data_view.abstract_data_model import AbstractDataModel
from pyface.data_view.abstract_value_type import AbstractValueType
from pyface.data_view.data_formats import (
    csv_column_format, csv_format, csv_row_format, from_csv, from_csv_column,
    from_csv_row, from_json, from_npy, html_format, npy_format,
    standard_text_format, to_csv, to_csv_column, to_csv_row, to_json, to_npy,
    table_format, text_column_format, text_row_format
)
from pyface.data_view.data_view_errors import (
    DataViewError, DataViewGetError, DataViewSetError
)
from pyface.data_view.i_data_view_widget import IDataViewWidget
from pyface.data_view.i_data_wrapper import (
    DataFormat, IDataWrapper, text_format
)
from pyface.data_view.index_manager import (
    AbstractIndexManager, IntIndexManager, TupleIndexManager
)


# ----------------------------------------------------------------------------
# Deferred imports
# ----------------------------------------------------------------------------

# These imports have the side-effect of performing toolkit selection

_toolkit_imports = {
    'DataViewWidget': 'data_view_widget',
    'DataWrapper': 'data_wrapper',
}


def __getattr__(name):
    """Lazily load attributes with side-effects

    In particular, lazily load toolkit backend names.  For efficiency, lazily
    loaded objects are injected into the module namespace
    """
    # sentinel object for no result
    not_found = object()
    result = not_found

    if name in _toolkit_imports:
        from pyface.toolkit import toolkit_object
        source = _toolkit_imports[name]
        result = toolkit_object(f"data_view.{source}:{name}")

    if result is not_found:
        raise AttributeError(f"module {__name__!r} has no attribute {name!r}")

    globals()[name] = result
    return result


# ----------------------------------------------------------------------------
# Introspection support
# ----------------------------------------------------------------------------

def __dir__():
    return sorted(set(globals()) | set(_toolkit_imports))



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""

API for the ``pyface.fields`` subpackage.

- :class:`~.CheckBoxField`
- :class:`~.ComboField`
- :class:`~.EditableField`
- :class:`~.Field`
- :class:`~.ImageField`
- :class:`~.LabelField`
- :class:`~.RadioButtonField`
- :class:`~.SpinField`
- :class:`~.TextField`
- :class:`~.TimeField`
- :class:`~.ToggleButtonField`

Interfaces
----------
- :class:`~.IComboField`
- :class:`~.IEditableField`
- :class:`~.IField`
- :class:`~.IImageField`
- :class:`~.ILabelField`
- :class:`~.ISpinField`
- :class:`~.ITextField`
- :class:`~.ITimeField`
- :class:`~.IToggleField`

"""

from .i_combo_field import IComboField
from .i_editable_field import IEditableField
from .i_field import IField
from .i_image_field import IImageField
from .i_label_field import ILabelField
from .i_spin_field import ISpinField
from .i_text_field import ITextField
from .i_time_field import ITimeField
from .i_toggle_field import IToggleField


# ----------------------------------------------------------------------------
# Deferred imports
# ----------------------------------------------------------------------------

# These imports have the side-effect of performing toolkit selection

_toolkit_imports = {
    'CheckBoxField': 'toggle_field',
    'ComboField': 'combo_field',
    'EditableField': 'editable_field',
    'Field': 'field',
    'ImageField': 'image_field',
    'LabelField': 'label_field',
    'RadioButtonField': 'toggle_field',
    'SpinField': 'spin_field',
    'TextField': 'text_field',
    'TimeField': 'time_field',
    'ToggleButtonField': 'toggle_field',
}


def __getattr__(name):
    """Lazily load attributes with side-effects

    In particular, lazily load toolkit backend names.  For efficiency, lazily
    loaded objects are injected into the module namespace
    """
    # sentinel object for no result
    not_found = object()
    result = not_found

    if name in _toolkit_imports:
        from pyface.toolkit import toolkit_object
        source = _toolkit_imports[name]
        result = toolkit_object(f"fields.{source}:{name}")

    if result is not_found:
        raise AttributeError(f"module {__name__!r} has no attribute {name!r}")

    globals()[name] = result
    return result


# ----------------------------------------------------------------------------
# Introspection support
# ----------------------------------------------------------------------------

def __dir__():
    return sorted(set(globals()) | set(_toolkit_imports))



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
# (C) Copyright 2007 Riverbank Computing Limited
# This software is provided without warranty under the terms of the BSD license.
# However, when used with the GPL version of PyQt the additional terms described in the PyQt GPL exception also apply

from pyface.qt import QtGui

from traits.api import provides

from pyface.i_heading_text import IHeadingText, MHeadingText
from .layout_widget import LayoutWidget


@provides(IHeadingText)
class HeadingText(MHeadingText, LayoutWidget):
    """ The Qt-specific implementation of a HeadingText.
    """

    # ------------------------------------------------------------------------
    # 'IWidget' interface.
    # ------------------------------------------------------------------------

    def _create_control(self, parent):
        """ Create the toolkit-specific control that represents the widget. """
        control = QtGui.QLabel(parent)
        control.setSizePolicy(
            QtGui.QSizePolicy.Policy.Preferred, QtGui.QSizePolicy.Policy.Fixed
        )
        return control

    # ------------------------------------------------------------------------
    # Private interface.
    # ------------------------------------------------------------------------

    def _set_control_text(self, text):
        """ Set the text on the toolkit specific widget. """
        # Bold the text. Qt supports a limited subset of HTML for rich text.
        text = f"<b>{text}</b>"
        self.control.setText(text)

    def _get_control_text(self):
        """ Get the text on the toolkit specific widget. """
        text = self.control.text()
        # remove the bolding from the text
        text = text[3:-4]
        return text



# (C) Copyright 2004-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Defines a wxPython ImageControl widget that is used by various Widgets.
"""


import wx


class ImageControl(wx.Window):
    """A wxPython control that displays an image.
    """

    def __init__(self, parent, bitmap=None, padding=10):
        """Initializes the object."""
        if bitmap is not None:
            size = wx.Size(
                bitmap.GetWidth() + padding, bitmap.GetHeight() + padding
            )
        else:
            size = wx.Size(32 + padding, 32 + padding)

        wx.Window.__init__(self, parent, -1, size=size,)
        self._bitmap = bitmap

        # Set up the 'paint' event handler:
        self.Bind(wx.EVT_PAINT, self._on_paint)

    def SetBitmap(self, bitmap=None):
        """Sets the bitmap image."""
        if bitmap is not None:
            if bitmap != self._bitmap:
                self._bitmap = bitmap
                self.Refresh()
        else:
            self._bitmap = None

    def GetBitmap(self):
        """Gets the bitmap image."""
        return self._bitmap

    def _on_paint(self, event=None):
        """Handles the control being re-painted."""
        wdc = wx.PaintDC(self)
        wdx, wdy = self.GetClientSize()
        bitmap = self._bitmap
        if bitmap is None:
            return
        bdx = bitmap.GetWidth()
        bdy = bitmap.GetHeight()
        wdc.DrawBitmap(bitmap, (wdx - bdx) // 2, (wdy - bdy) // 2, True)



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" A dialog that allows the user to select a color. """

from pyface.qt import QtGui

from traits.api import Bool, provides

from pyface.color import Color
from pyface.ui_traits import PyfaceColor
from pyface.i_color_dialog import IColorDialog
from .dialog import Dialog


@provides(IColorDialog)
class ColorDialog(Dialog):
    """ A dialog that allows the user to choose a color.
    """

    # 'IColorDialog' interface ----------------------------------------------

    #: The color in the dialog.
    color = PyfaceColor()

    #: Whether or not to allow the user to chose an alpha value.
    show_alpha = Bool(False)

    # ------------------------------------------------------------------------
    # 'IDialog' interface.
    # ------------------------------------------------------------------------

    def _create_contents(self, parent):
        # In PyQt this is a canned dialog so there are no contents.
        pass

    # ------------------------------------------------------------------------
    # 'IWindow' interface.
    # ------------------------------------------------------------------------

    def close(self):
        if self.control.result() == QtGui.QDialog.DialogCode.Accepted:
            qcolor = self.control.selectedColor()
            self.color = Color.from_toolkit(qcolor)
        return super(ColorDialog, self).close()

    # ------------------------------------------------------------------------
    # 'IWindow' interface.
    # ------------------------------------------------------------------------

    def _create_control(self, parent):
        qcolor = self.color.to_toolkit()
        dialog = QtGui.QColorDialog(qcolor, parent)
        if self.show_alpha:
            dialog.setOptions(QtGui.QColorDialog.ColorDialogOption.ShowAlphaChannel)
        return dialog



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traits.api import Bool, Str, Tuple

from pyface.action.i_action_manager import IActionManager
from pyface.ui_traits import Orientation


class IToolBarManager(IActionManager):
    """ The interface for a tool bar manager. """

    # 'IToolBarManager' interface -------------------------------------------

    # The size of tool images (width, height).
    image_size = Tuple((16, 16))

    # The toolbar name (used to distinguish multiple toolbars).
    name = Str("ToolBar")

    # The orientation of the toolbar.
    orientation = Orientation("horizontal")

    # Should we display the name of each tool bar tool under its image?
    show_tool_names = Bool(True)

    # Should we display the horizontal divider?
    show_divider = Bool(True)

    # ------------------------------------------------------------------------
    # 'ToolBarManager' interface.
    # ------------------------------------------------------------------------

    def create_tool_bar(self, parent, controller=None):
        """ Creates a tool bar.

        Parameters
        ----------
        parent : toolkit control
            The toolkit control that owns the toolbar.
        controller : pyface.action.action_controller.ActionController
            An optional ActionController for all items in the toolbar.
        """



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Helper functions for working with images

This module provides helper functions for converting between numpy arrays
and Qt QImages, as well as between the various image types in a standardized
way.
"""

from enum import Enum

from pyface.qt import qt_api
from pyface.qt.QtCore import Qt
from pyface.qt.QtGui import QImage, QPixmap, QIcon


class ScaleMode(Enum):
    fast = Qt.TransformationMode.FastTransformation
    smooth = Qt.TransformationMode.SmoothTransformation


class AspectRatio(Enum):
    ignore = Qt.AspectRatioMode.IgnoreAspectRatio
    keep_constrain = Qt.AspectRatioMode.KeepAspectRatio
    keep_expand = Qt.AspectRatioMode.KeepAspectRatioByExpanding


def image_to_bitmap(image):
    """ Convert a QImage to a QPixmap.
    Parameters
    ----------
    image : QImage
        The QImage to convert.

    Return
    ------
    bitmap : QPixmap
        The corresponding QPixmap.
    """
    bitmap = QPixmap.fromImage(image)
    # keep a reference to the QImage to ensure underlying data is available
    bitmap._image = image
    return bitmap


def bitmap_to_image(bitmap):
    """ Convert a QPixmap to a QImage.
    Parameters
    ----------
    bitmap : QPixmap
        The QPixmap to convert.

    Return
    ------
    image : QImage
        The corresponding QImage.
    """
    return bitmap.toImage()


def bitmap_to_icon(bitmap):
    """ Convert a QPixmap to a QIcon.
    Parameters
    ----------
    bitmap : QPixmap
        The QPixmap to convert.

    Return
    ------
    icon : QIcon
        The corresponding QIcon.
    """
    return QIcon(bitmap)


def resize_image(image, size, aspect_ratio=AspectRatio.ignore,
                 mode=ScaleMode.fast):
    """ Resize a toolkit image to the given size. """
    return image.scaled(*size, aspect_ratio.value, mode.value)


def resize_bitmap(bitmap, size, aspect_ratio=AspectRatio.ignore,
                  mode=ScaleMode.fast):
    """ Resize a toolkit bitmap to the given size. """
    return bitmap.scaled(*size, aspect_ratio.value, mode.value)


def image_to_array(image):
    """ Convert a QImage to a numpy array.

    This copies the data returned from Qt.

    Parameters
    ----------
    image : QImage
        The QImage that we want to extract the values from.  The format must
        be either RGB32 or ARGB32.

    Return
    ------
    array : ndarray
        An N x M x 4 array of unsigned 8-bit ints as RGBA values.
    """
    import numpy as np

    width, height = image.width(), image.height()
    channels = image.pixelFormat().channelCount()
    data = image.bits()
    if qt_api in {'pyqt', 'pyqt5'}:
        data = data.asarray(width * height * channels)
    array = np.array(data, dtype='uint8')
    array.shape = (height, width, channels)
    if image.format() in {QImage.Format.Format_RGB32, QImage.Format.Format_ARGB32}:
        # comes in as BGRA, but want RGBA
        array = array[:, :, [2, 1, 0, 3]]
    else:
        raise ValueError(
            "Unsupported QImage format {}".format(image.format())
        )
    return array


def array_to_image(array):
    """ Convert a numpy array to a QImage.

    This copies the data before passing it to Qt.

    Parameters
    ----------
    array : ndarray
        An N x M x {3, 4} array of unsigned 8-bit ints.  The image
        format is assumed to be RGB or RGBA, based on the shape.

    Return
    ------
    image : QImage
        The QImage created from the data.  The pixel format is
        QImage.Format.Format_RGB32.
    """
    import numpy as np

    if array.ndim != 3:
        raise ValueError("Array must be either RGB or RGBA values.")

    height, width, channels = array.shape
    data = np.empty((height, width, 4), dtype='uint8')
    if channels == 3:
        data[:, :, [2, 1, 0]] = array
        data[:, :, 3] = 0xff
    elif channels == 4:
        data[:, :, [2, 1, 0, 3]] = array
    else:
        raise ValueError("Array must be either RGB or RGBA values.")

    bytes_per_line = 4 * width

    if channels == 3:
        image = QImage(data.data, width, height, bytes_per_line,
                       QImage.Format.Format_RGB32)

    elif channels == 4:
        image = QImage(data.data, width, height, bytes_per_line,
                       QImage.Format.Format_ARGB32)
    # keep a reference to the array to ensure underlying data is available
    image._numpy_data = data
    return image



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The interface for a dialog that allows the user to select a color. """

from pyface.ui_traits import PyfaceFont
from pyface.i_dialog import IDialog


class IFontDialog(IDialog):
    """ The interface for a dialog that allows the user to choose a font.
    """

    # 'IFontDialog' interface ---------------------------------------------#

    #: The font in the dialog.
    font = PyfaceFont()



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

import unittest

from traits.api import HasStrictTraits, TraitError

from pyface.font import Font, FontSize, FontStretch, SIZES, STRETCHES
from pyface.toolkit import toolkit_object


class FontSizeDummy(HasStrictTraits):

    size = FontSize()
    size_default_1 = FontSize(14.0)
    size_default_2 = FontSize("14.0")
    size_default_3 = FontSize("14.0pt")
    size_default_4 = FontSize("large")


class TestFontSizeTrait(unittest.TestCase):

    def test_font_size_trait_defaults(self):
        dummy = FontSizeDummy()

        self.assertEqual(dummy.size, 12.0)
        self.assertEqual(dummy.size_default_1, 14.0)
        self.assertEqual(dummy.size_default_2, 14.0)
        self.assertEqual(dummy.size_default_3, 14.0)
        self.assertEqual(dummy.size_default_4, 14.0)

    def test_font_sizes(self):
        dummy = FontSizeDummy()
        for size, value in SIZES.items():
            with self.subTest(size=size):
                dummy.size = size
                self.assertEqual(dummy.size, value)

    def test_font_size_trait_invalid_default(self):
        for size in ["badvalue", -1.0, "-1.0", "0pt"]:
            with self.subTest(size=size):
                with self.assertRaises(TraitError):
                    FontSize(size)

    def test_font_size_trait_validate(self):
        dummy = FontSizeDummy()
        for size, expected in [
            (14.0, 14.0),
            ("15.0", 15.0),
            ("16pt", 16.0),
            ("17.0px", 17.0),
        ]:
            with self.subTest(size=size):
                dummy.size = size
                self.assertEqual(dummy.size, expected)

    def test_font_size_trait_invalid_validate(self):
        dummy = FontSizeDummy()
        for size in ["badvalue", -1.0, "-1.0", "0pt"]:
            with self.subTest(size=size):
                with self.assertRaises(TraitError):
                    dummy.size = size


class FontStretchDummy(HasStrictTraits):

    stretch = FontStretch()
    stretch_default_1 = FontStretch(150)
    stretch_default_2 = FontStretch("150.0")
    stretch_default_3 = FontStretch("150.0%")
    stretch_default_4 = FontStretch("expanded")


class TestFontStretchTrait(unittest.TestCase):

    def test_font_stretch_trait_defaults(self):
        dummy = FontStretchDummy()

        self.assertEqual(dummy.stretch, 100.0)
        self.assertEqual(dummy.stretch_default_1, 150.0)
        self.assertEqual(dummy.stretch_default_2, 150.0)
        self.assertEqual(dummy.stretch_default_3, 150.0)
        self.assertEqual(dummy.stretch_default_4, 125.0)

    def test_font_stretches(self):
        dummy = FontStretchDummy()
        for stretch, value in STRETCHES.items():
            with self.subTest(stretch=stretch):
                dummy.stretch = stretch
                self.assertEqual(dummy.stretch, value)

    def test_font_stretch_trait_invalid_default(self):
        for stretch in ["badvalue", 49.5, "49.5", "49.5%", 200.1]:
            with self.subTest(stretch=stretch):
                with self.assertRaises(TraitError):
                    FontStretch(stretch)

    def test_font_stretch_trait_validate(self):
        dummy = FontStretchDummy()

        for stretch, expected in [
            (150.0, 150.0),
            ("125", 125.0),
            ("50%", 50.0),
            ("ultra-expanded", 200.0)
        ]:
            with self.subTest(stretch=stretch):
                dummy.stretch = stretch
                self.assertEqual(dummy.stretch, expected)

    def test_font_stretch_trait_invalid_validate(self):
        dummy = FontStretchDummy()
        for stretch in ["badvalue", 49.5, "200.1", "49.9%"]:
            with self.subTest(stretch=stretch):
                with self.assertRaises(TraitError):
                    dummy.stretch = stretch


class TestFont(unittest.TestCase):

    def test_default(self):
        font = Font()

        self.assertEqual(font.family, ['default'])
        self.assertEqual(font.size, 12.0)
        self.assertEqual(font.weight, 'normal')
        self.assertEqual(font.stretch, 100)
        self.assertEqual(font.style, 'normal')
        self.assertEqual(font.variants, set())

    def test_typical(self):
        font = Font(
            family=['Helvetica', 'sans-serif'],
            size='large',
            weight='demi-bold',
            stretch='condensed',
            style='italic',
            variants={'small-caps'},
            decorations={'underline'},
        )

        self.assertEqual(font.family, ['Helvetica', 'sans-serif'])
        self.assertEqual(font.size, 14.0)
        self.assertEqual(font.weight, 'demi-bold')
        self.assertEqual(font.weight_, 600)
        self.assertEqual(font.stretch, 75)
        self.assertEqual(font.style, 'italic')
        self.assertEqual(font.variants, {'small-caps'})
        self.assertEqual(font.decorations, {'underline'})

    def test_family_sequence(self):
        font = Font(family=('Helvetica', 'sans-serif'))
        self.assertEqual(font.family, ['Helvetica', 'sans-serif'])

    def test_variants_frozenset(self):
        font = Font(variants=frozenset({'small-caps'}))
        self.assertEqual(font.variants, {'small-caps'})

    def test_decorations_frozenset(self):
        font = Font(decorations=frozenset({'underline'}))
        self.assertEqual(font.decorations, {'underline'})

    def test_str(self):
        font = Font()

        description = str(font)

        self.assertEqual(description, "12pt default")

    def test_str_typical(self):
        font = Font(
            family=['Comic Sans', 'decorative'],
            size='large',
            weight='demi-bold',
            stretch='condensed',
            style='italic',
            variants={'small-caps'},
            decorations={'underline'},
        )

        description = str(font)

        self.assertEqual(
            description,
            "italic small-caps underline demi-bold 75% 14pt "
            "'Comic Sans', decorative"
        )

    def test_repr(self):
        font = Font()

        text = repr(font)

        # this is little more than a smoke check, but good enough
        self.assertTrue(text.startswith('Font('))

    def test_repr_typical(self):
        font = Font(
            family=['Helvetica', 'sans-serif'],
            size='large',
            weight='demi-bold',
            stretch='condensed',
            style='italic',
            variants={'small-caps'},
            decorations={'underline'},
        )

        text = repr(font)

        # this is little more than a smoke check, but good enough
        self.assertTrue(text.startswith('Font('))

    def test_to_toolkit(self):
        font = Font()

        # smoke test
        toolkit_font = font.to_toolkit()
        self.assertIsNotNone(toolkit_font)

    def test_to_toolkit_typical(self):
        font = Font(
            family=['Helvetica', 'sans-serif'],
            size='large',
            weight='demi-bold',
            stretch='condensed',
            style='italic',
            variants={'small-caps'},
            decorations={'underline', 'strikethrough', 'overline'},
        )

        # smoke test
        toolkit_font = font.to_toolkit()
        self.assertIsNotNone(toolkit_font)

    def test_toolkit_default_roundtrip(self):
        font = Font()

        # smoke test
        result = Font.from_toolkit(font.to_toolkit())

        # defaults should round-trip
        self.assertTrue(result.family[-1], 'default')
        self.assertEqual(result.size, font.size)
        self.assertEqual(result.weight, font.weight)
        self.assertEqual(result.stretch, font.stretch)
        self.assertEqual(result.variants, font.variants)
        self.assertEqual(result.decorations, font.decorations)

    def test_from_toolkit_typical(self):
        font = Font(
            family=['Helvetica', 'sans-serif'],
            size='large',
            weight='bold',
            stretch='condensed',
            style='italic',
            variants={'small-caps'},
            decorations={'underline', 'strikethrough', 'overline'},
        )

        # smoke test
        result = Font.from_toolkit(font.to_toolkit())

        # we expect some things should round-trip no matter what system
        self.assertEqual(result.size, font.size)
        self.assertEqual(result.weight, font.weight)
        self.assertEqual(result.style, font.style)

    def test_toolkit_font_to_properties(self):
        toolkit_font_to_properties = toolkit_object(
            'font:toolkit_font_to_properties')

        font = Font(
            family=['Helvetica', 'sans-serif'],
            size='large',
            weight='demi-bold',
            stretch='condensed',
            style='italic',
            variants={'small-caps'},
            decorations={'underline', 'strikethrough', 'overline'},
        )

        properties = toolkit_font_to_properties(font.to_toolkit())

        self.assertEqual(
            set(properties.keys()),
            {
                'family', 'size', 'stretch', 'weight', 'style', 'variants',
                'decorations'
            }
        )



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


""" The interface for a menu bar manager. """

from pyface.action.i_action_manager import IActionManager


class IMenuBarManager(IActionManager):
    """ The interface for a menu bar manager. """

    # ------------------------------------------------------------------------
    # 'MenuBarManager' interface.
    # ------------------------------------------------------------------------

    def create_menu_bar(self, parent, controller=None):
        """ Creates a menu bar representation of the manager.

        Parameters
        ----------
        parent : toolkit control
            The toolkit control that owns the menubar.
        controller : pyface.action.action_controller.ActionController
            An optional ActionController for all items in the menubar.
        """



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from importlib import import_module
from importlib.abc import MetaPathFinder, Loader
from importlib.machinery import ModuleSpec
from importlib.util import find_spec
import sys


# Import hooks for loading pyface.ui.qt.* in place of a pyface.ui.qt4.*
# This is just the implementation, it is not connected in this module, but
# is available for applications which want to install it themselves.
# It is here rather than in pyface.ui.qt4 so it can be imported and used
# without generating the warnings from pyface.ui.qt4
#
# To use manually:
#
#     import sys
#     sys.meta_path.append(ShadowedModuleFinder())

class ShadowedModuleLoader(Loader):
    """This loads another module into sys.modules with a given name.

    Parameters
    ----------
    fullname : str
        The full name of the module we're trying to import.
        Eg. "pyface.ui.qt4.foo"
    new_name : str
        The full name of the corresponding "real" module.
        Eg. "pyface.ui.qt.foo"
    new_spec : ModuleSpec
        The spec object for the corresponding "real" module.
    """

    def __init__(self, fullname, new_name, new_spec):
        self.fullname = fullname
        self.new_name = new_name
        self.new_spec = new_spec

    def create_module(self, spec):
        """Create the module object.

        This doesn't create the module object directly, rather it gets the
        underlying "real" module's object, importing it if needed.  This object
        is then returned as the "new" module.
        """
        if self.new_name not in sys.modules:
            import_module(self.new_name)
        return sys.modules[self.new_name]

    def exec_module(self, module):
        """Execute code for the module.

        This is given a module which has already been executed, so we don't
        need to execute anything.  However we do need to remove the __spec__
        that the importlibs machinery has injected into the module and
        replace it with the original spec for the underlying "real" module.
        """
        # patch up the __spec__ with the true module's original __spec__
        if self.new_spec:
            module.__spec__ = self.new_spec
            self.new_spec = None


class ShadowedModuleFinder(MetaPathFinder):
    """MetaPathFinder for shadowing modules in a package

    This finds modules with names that match a package but arranges loading
    from a different package.  By default this is  matches imports from any
    path starting with pyface.ui.qt4. and returns a loader which will instead
    load from pyface.ui.qt.*

    The end result is that sys.modules has two entries for pointing to the
    same module object.

    This may be hooked up by code in pyface.ui.qt4, but it can also be
    installed manually with::

        import sys
        sys.meta_path.append(ShadowedModuleFinder())

    Parameters
    ----------
    package : str
        The prefix of the "shadow" package.
    true_package : str
        The prefix of the "real" package which contains the actual code.
    """

    def __init__(self, package="pyface.ui.qt4.", true_package="pyface.ui.qt."):
        self.package = package
        self.true_package = true_package

    def find_spec(self, fullname, path, target=None):
        if fullname.startswith(self.package):
            new_name = fullname.replace(self.package, self.true_package, 1)
            new_spec = find_spec(new_name)
            if new_spec is None:
                return None
            return ModuleSpec(
                name=fullname,
                loader=ShadowedModuleLoader(fullname, new_name, new_spec),
                is_package=(new_spec.submodule_search_locations is not None),
            )



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


"""

API for the ``pyface.action`` subpackage.

Actions
-------

- :class:`~.Action`
- :class:`~.FieldAction`
- :class:`~.GUIApplicationAction`
- :class:`~.ListeningAction`
- :class:`~.TraitsUIWidgetAction`
- :class:`~.WindowAction`

Action Controller
-----------------

- :class:`~pyface.action.action_controller.ActionController`

Action Event
------------

- :class:`~.ActionEvent`

Action Managers
---------------

- :class:`~.ActionManager`
- ``MenuManager``
- ``MenuBarManager``
- ``StatusBarManager``
- ``ToolBarManager``

Action Manager Items
--------------------

- :class:`~.ActionManagerItem`
- :class:`~.ActionItem`

Layout support
--------------

- :class:`~.Group`
- :class:`~.Separator`

Useful Application and Window actions
-------------------------------------

- :class:`~.AboutAction`
- :class:`~.CloseActiveWindowAction`
- :class:`~.CreateWindowAction`
- :class:`~.ExitAction`
- :class:`~.CloseWindowAction`

"""

# Imports which don't select the toolkit as a side-effect.

from .action import Action
from .action_controller import ActionController
from .action_event import ActionEvent
from .action_manager import ActionManager
from .action_manager_item import ActionManagerItem
from .field_action import FieldAction
from .group import Group, Separator
from .gui_application_action import (
    AboutAction,
    CloseActiveWindowAction,
    CreateWindowAction,
    ExitAction,
    GUIApplicationAction,
)
from .i_action_manager import IActionManager
from .i_menu_bar_manager import IMenuBarManager
from .i_menu_manager import IMenuManager
from .i_status_bar_manager import IStatusBarManager
from .i_tool_bar_manager import IToolBarManager
from .listening_action import ListeningAction
from .traitsui_widget_action import TraitsUIWidgetAction
from .window_action import CloseWindowAction, WindowAction


# ----------------------------------------------------------------------------
# Deferred imports
# ----------------------------------------------------------------------------

# These imports have the side-effect of performing toolkit selection

_toolkit_imports = {
    'MenuManager': "menu_manager",
    'MenuBarManager': "menu_bar_manager",
    'StatusBarManager': "status_bar_manager",
    'ToolBarManager': "tool_bar_manager",
}

# These are pyface.* imports that have selection as a side-effect
# TODO: refactor to delay imports where possible
_relative_imports = {
    'ActionItem': "action_item",
}


def __getattr__(name):
    """Lazily load attributes with side-effects

    In particular, lazily load toolkit backend names.  For efficiency, lazily
    loaded objects are injected into the module namespace
    """
    # sentinel object for no result
    not_found = object()
    result = not_found

    if name in _relative_imports:
        from importlib import import_module
        source = _relative_imports[name]
        module = import_module(f"pyface.action.{source}")
        result = getattr(module, name)

    elif name in _toolkit_imports:
        from pyface.toolkit import toolkit_object
        source = _toolkit_imports[name]
        result = toolkit_object(f"action.{source}:{name}")

    if result is not_found:
        raise AttributeError(f"module {__name__!r} has no attribute {name!r}")

    globals()[name] = result
    return result


# ----------------------------------------------------------------------------
# Introspection support
# ----------------------------------------------------------------------------

# the list of available names we report for introspection purposes
_extra_names = set(_toolkit_imports) | set(_relative_imports)


def __dir__():
    return sorted(set(globals()) | _extra_names)



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The interface for a dialog that allows the user to select a color. """

import wx

from traits.api import Bool, provides

from pyface.color import Color
from pyface.ui_traits import PyfaceColor
from pyface.i_color_dialog import IColorDialog
from .dialog import Dialog

# The WxPython version in a convenient to compare form.
wx_version = tuple(int(x) for x in wx.__version__.split('.')[:3])


@provides(IColorDialog)
class ColorDialog(Dialog):
    """ A dialog for selecting colors.
    """

    # 'IColorDialog' interface ----------------------------------------------

    #: The color in the dialog.
    color = PyfaceColor()

    #: Whether or not to allow the user to chose an alpha value.  Only works
    #: for wxPython 4.1 and higher.
    show_alpha = Bool(False)

    # ------------------------------------------------------------------------
    # 'IDialog' interface.
    # ------------------------------------------------------------------------

    def _create_contents(self, parent):
        # In wx this is a canned dialog.
        pass

    # ------------------------------------------------------------------------
    # 'IWindow' interface.
    # ------------------------------------------------------------------------

    def close(self):
        colour_data = self.control.GetColourData()
        wx_colour = colour_data.GetColour()
        self.color = Color.from_toolkit(wx_colour)
        super(ColorDialog, self).close()

    # ------------------------------------------------------------------------
    # 'IWidget' interface.
    # ------------------------------------------------------------------------

    def _create_control(self, parent):
        wx_colour = self.color.to_toolkit()
        data = wx.ColourData()
        data.SetColour(wx_colour)
        if wx_version >= (4, 1):
            data.SetChooseAlpha(self.show_alpha)
        dialog = wx.ColourDialog(parent, data)
        return dialog



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The Wx-specific implementation of the text field class """


from traits.api import provides

from pyface.fields.i_editable_field import IEditableField, MEditableField
from pyface.ui.wx.fields.field import Field


@provides(IEditableField)
class EditableField(MEditableField, Field):
    """ The Wx-specific implementation of the EditableField class

    This is an abstract class which is not meant to be instantiated.
    """



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Utility functions for handling Qt dates and times. """

import datetime
import unittest

from pyface.qt.QtCore import QTime

from ..datetime import pytime_to_qtime, qtime_to_pytime


class TestTimeConversion(unittest.TestCase):

    def test_pytime_to_qtime(self):
        pytime = datetime.time(9, 8, 7, 123456)

        qtime = pytime_to_qtime(pytime)

        self.assertEqual(qtime.hour(), 9)
        self.assertEqual(qtime.minute(), 8)
        self.assertEqual(qtime.second(), 7)
        self.assertEqual(qtime.msec(), 123)

    def test_qtime_to_pytime(self):
        qtime = QTime(9, 8, 7, 123)

        pytime = qtime_to_pytime(qtime)

        self.assertEqual(pytime.hour, 9)
        self.assertEqual(pytime.minute, 8)
        self.assertEqual(pytime.second, 7)
        self.assertEqual(pytime.microsecond, 123000)



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from pyface.testing.widget_mixin import WidgetMixin


class LayoutWidgetMixin(WidgetMixin):
    """ Test mixin for classes which inherit LayoutWidget. """

    def test_minimum_size(self):
        # create a widget with a minimum size
        self.widget.minimum_size = (100, 100)
        self.widget.create()
        minimum_size = self.widget._get_control_minimum_size()
        self.gui.process_events()
        self.assertEqual(minimum_size, (100, 100))

        # change the minimum size
        self.widget.minimum_size = (50, 50)
        self.gui.process_events()
        minimum_size = self.widget._get_control_minimum_size()

        self.assertEqual(minimum_size, (50, 50))

    def test_maximum_size(self):
        # create a widget with a maximum size
        self.widget.maximum_size = (1000, 1000)
        self.widget.create()
        maximum_size = self.widget._get_control_maximum_size()
        self.gui.process_events()
        self.assertEqual(maximum_size, (1000, 1000))

        # change the maximum size
        self.widget.maximum_size = (50, 50)
        self.gui.process_events()
        maximum_size = self.widget._get_control_maximum_size()

        self.assertEqual(maximum_size, (50, 50))

    def test_stretch(self):
        # create a widget with a maximum size
        self.widget.stretch = (2, 3)
        self.widget.create()
        stretch = self.widget._get_control_stretch()
        self.gui.process_events()
        self.assertEqual(stretch, (2, 3))

        # change the maximum size
        self.widget.stretch = (5, 0)
        self.gui.process_events()
        maximum_size = self.widget._get_control_stretch()

        self.assertEqual(maximum_size, (5, 0))

    def test_size_policy(self):
        # create a widget with a maximum size
        self.widget.create()
        self.assertEqual(
            self.widget._get_control_size_policy(),
            ("default", "default"),
        )

        for horizontal in ["fixed", "preferred", "expand"]:
            for vertical in ["fixed", "preferred", "expand"]:
                with self.subTest(horizontal=horizontal, vertical=vertical):
                    self.widget.size_policy = (horizontal, vertical)
                    self.gui.process_events()
                    self.assertEqual(
                        self.widget._get_control_size_policy(),
                        (horizontal, vertical),
                    )



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from contextlib import contextmanager
import os
import sys
import unittest

from traits.etsconfig.api import ETSConfig

from pyface.ui import ShadowedModuleFinder


class TestQt4ImportHooks(unittest.TestCase):

    def test_qt4_import_no_hook(self):
        with self._unload_modules([
            "pyface.ui.qt4",
            "pyface.ui.qt4.tests",
            "pyface.ui.qt4.tests.good_package",
            "pyface.ui.qt4.tests.good_package.good_import",
        ]):
            with self._clean_meta_path():
                with self.assertWarns(FutureWarning):
                    with self.assertRaises(ModuleNotFoundError) as cm:
                        import pyface.ui.qt4.tests.good_package.good_import  # noqa F401

                    self.assertEqual(cm.exception.name, "pyface.ui.qt4.tests")

    def test_qt4_import_other_package_hook(self):
        with self._unload_modules([
            "pyface.ui.qt4",
            "pyface.ui.qt4.tests",
            "pyface.ui.qt4.tests.good_package",
            "pyface.ui.qt4.tests.good_package.good_import",
        ]):
            with self._clean_meta_path():
                # test with shadow module finder for different package
                # (eg. another ETS package using the same code)
                sys.meta_path.append(ShadowedModuleFinder(
                    package='foo.bar.',
                    true_package='foo.baz.',
                ))
                with self.assertWarns(FutureWarning):
                    with self.assertRaises(ModuleNotFoundError) as cm:
                        import pyface.ui.qt4.tests.good_package.good_import  # noqa F401

                    self.assertEqual(cm.exception.name, "pyface.ui.qt4.tests")

    def test_qt4_import_with_hook(self):
        with self._unload_modules([
            "pyface.ui.qt4",
            "pyface.ui.qt4.tests",
            "pyface.ui.qt4.tests.good_package",
            "pyface.ui.qt4.tests.good_package.good_import",
        ]):
            with self._clean_meta_path():
                sys.meta_path.append(ShadowedModuleFinder())

                with self.assertWarns(DeprecationWarning):
                    import pyface.ui.qt4.tests.good_package.good_import  # noqa F401

                import pyface.ui.qt.tests.good_package.good_import

                self.assertIs(
                    pyface.ui.qt4.tests.good_package.good_import,
                    pyface.ui.qt.tests.good_package.good_import,
                )

                self.assertIs(
                    pyface.ui.qt4.tests.good_package,
                    pyface.ui.qt.tests.good_package,
                )

                self.assertIs(
                    pyface.ui.qt4.tests,
                    pyface.ui.qt.tests,
                )

    def test_qt4_import_with_hook_no_module(self):
        with self._unload_modules([
            "pyface.ui.qt4",
            "pyface.ui.qt4.tests",
            "pyface.ui.qt4.tests.good_package",
        ]):
            with self._clean_meta_path():
                sys.meta_path.append(ShadowedModuleFinder())

                with self.assertWarns(DeprecationWarning):
                    with self.assertRaises(ModuleNotFoundError) as cm:
                        import pyface.ui.qt4.tests.good_package.no_module  # noqa F401

                    self.assertEqual(
                        cm.exception.name,
                        "pyface.ui.qt4.tests.good_package.no_module",
                    )

    def test_qt4_import_with_hook_module_has_bad_import(self):
        with self._unload_modules([
            "pyface.ui.qt4",
            "pyface.ui.qt4.tests",
            "pyface.ui.qt4.tests.good_package",
        ]):
            with self._clean_meta_path():
                sys.meta_path.append(ShadowedModuleFinder())

                with self.assertWarns(DeprecationWarning):
                    with self.assertRaises(ModuleNotFoundError) as cm:
                        import pyface.ui.qt4.tests.good_package.has_bad_import  # noqa F401

                    self.assertEqual(cm.exception.name, "nonexistent_module")

    def test_qt4_import_with_ets_qt4_imports(self):
        with self._unload_modules([
            "pyface.ui.qt4",
            "pyface.ui.qt4.tests",
            "pyface.ui.qt4.tests.good_package",
            "pyface.ui.qt4.tests.good_package.good_import",
        ]):
            with self._clean_meta_path():
                with self._set_environment("ETS_QT4_IMPORTS", "1"):

                    with self.assertWarns(DeprecationWarning):
                        import pyface.ui.qt4.tests.good_package.good_import

                    import pyface.ui.qt.tests.good_package.good_import

                    self.assertIs(
                        pyface.ui.qt4.tests.good_package.good_import,
                        pyface.ui.qt.tests.good_package.good_import,
                    )

    def test_qt4_import_with_ets_toolkit_qt4(self):
        with self._unload_modules([
            "pyface.ui.qt4",
            "pyface.ui.qt4.tests",
            "pyface.ui.qt4.tests.good_package",
            "pyface.ui.qt4.tests.good_package.good_import",
        ]):
            with self._clean_meta_path():
                with self._set_environment("ETS_TOOLKIT", "qt4"):

                    with self.assertWarns(DeprecationWarning):
                        import pyface.ui.qt4.tests.good_package.good_import

                    import pyface.ui.qt.tests.good_package.good_import

                    self.assertIs(
                        pyface.ui.qt4.tests.good_package.good_import,
                        pyface.ui.qt.tests.good_package.good_import,
                    )

    def test_qt4_import_with_ets_toolkit_qt(self):
        with self._unload_modules([
            "pyface.ui.qt4",
            "pyface.ui.qt4.tests",
            "pyface.ui.qt4.tests.good_package",
            "pyface.ui.qt4.tests.good_package.good_import",
        ]):
            with self._clean_meta_path():
                with self._set_environment("ETS_TOOLKIT", "qt"):
                    with self.assertWarns(FutureWarning):
                        with self.assertRaises(ModuleNotFoundError):
                            import pyface.ui.qt4.tests.good_package.good_import  # noqa F401

    def test_qt4_import_with_etsconfig_toolkit_qt4(self):
        with self._unload_modules([
            "pyface.ui.qt4",
            "pyface.ui.qt4.tests",
            "pyface.ui.qt4.tests.good_package",
            "pyface.ui.qt4.tests.good_package.good_import",
        ]):
            with self._clean_meta_path():
                with self._set_etsconfig_toolkit("qt4"):

                    with self.assertWarns(DeprecationWarning):
                        import pyface.ui.qt4.tests.good_package.good_import

                    import pyface.ui.qt.tests.good_package.good_import

                    self.assertIs(
                        pyface.ui.qt4.tests.good_package.good_import,
                        pyface.ui.qt.tests.good_package.good_import,
                    )

    def test_qt4_import_with_etsconfig_toolkit_qt(self):
        with self._unload_modules([
            "pyface.ui.qt4",
            "pyface.ui.qt4.tests",
            "pyface.ui.qt4.tests.good_package",
            "pyface.ui.qt4.tests.good_package.good_import",
        ]):
            with self._clean_meta_path():
                with self._set_etsconfig_toolkit("qt"):
                    with self.assertWarns(FutureWarning):
                        with self.assertRaises(ModuleNotFoundError):
                            import pyface.ui.qt4.tests.good_package.good_import  # noqa F401

    @contextmanager
    def _clean_meta_path(self):
        """Temporarily remove ShadowedModuleFinder instances from sys.meta_path"""
        old_meta_path = sys.meta_path[:]
        sys.meta_path[:] = [
            finder for finder in sys.meta_path
            if not isinstance(finder, ShadowedModuleFinder)
        ]
        try:
            yield
        finally:
            sys.meta_path[:] = old_meta_path

    @contextmanager
    def _unload_modules(self, names):
        """Temporarily remove listed modules from sys.modules"""
        old_modules = {
            name: sys.modules.pop(name, None)
            for name in names
        }
        try:
            yield
        finally:
            for name, old_module in old_modules.items():
                if old_module:
                    sys.modules[name] = old_module
                else:
                    sys.modules.pop(name, None)

    @contextmanager
    def _set_environment(self, name, value):
        """Temporarily set an environment variable to a value"""
        old_value = os.environ.get(name, None)
        os.environ[name] = value
        try:
            yield
        finally:
            if old_value is None:
                del os.environ[name]
            else:
                os.environ[name] = old_value

    @contextmanager
    def _set_etsconfig_toolkit(self, value):
        """Temporarily set ETSConfig.toolkit to a value"""
        old_value = ETSConfig._toolkit
        ETSConfig._toolkit = value
        try:
            yield
        finally:
            ETSConfig._toolkit = old_value



# Copyright (c) 2005-2023, Enthought Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
import unittest

from traits.api import HasTraits, provides

from pyface.i_gui import IGUI
from pyface.ui.qt.util.event_loop_helper import EventLoopHelper


@provides(IGUI)
class DummyGUI(HasTraits):
    pass


class TestEventLoopHelper(unittest.TestCase):

    def test_gui_trait_expects_IGUI_interface(self):
        # Trivial test where we simply set the trait
        # and the test passes because no errors are raised.
        event_loop_helper = EventLoopHelper()
        event_loop_helper.gui = DummyGUI()



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import unittest

from ..heading_text import HeadingText
from ..split_panel import SplitPanel
from ..toolkit import toolkit_object
from ..window import Window

GuiTestAssistant = toolkit_object("util.gui_test_assistant:GuiTestAssistant")
no_gui_test_assistant = GuiTestAssistant.__name__ == "Unimplemented"


@unittest.skipIf(no_gui_test_assistant, "No GuiTestAssistant")
class TestSplitPanel(unittest.TestCase, GuiTestAssistant):
    def setUp(self):
        GuiTestAssistant.setUp(self)
        self.window = Window()
        self.window.create()

    def tearDown(self):
        if self.widget.control is not None:
            with self.delete_widget(self.widget.control):
                self.widget.destroy()

        if self.window.control is not None:
            with self.delete_widget(self.window.control):
                self.window.destroy()

        del self.widget
        del self.window
        GuiTestAssistant.tearDown(self)

    def test_lifecycle(self):
        # test that create/destroy works
        self.widget = SplitPanel(self.window.control)

        self.assertIsNone(self.widget.control)

        with self.event_loop():
            self.widget.create(parent=self.window.control)

        self.assertIsNotNone(self.widget.control)

        with self.event_loop():
            self.widget.destroy()

    def test_one_stage_create(self):
        # test that automatic creation works
        with self.event_loop():
            with self.assertWarns(DeprecationWarning):
                self.widget = SplitPanel(self.window.control, create=True)

        self.assertIsNotNone(self.widget.control)

        with self.event_loop():
            self.widget.destroy()

    def test_two_stage_create(self):
        # test that create=False works
        with self.assertWarns(DeprecationWarning):
            self.widget = SplitPanel(create=False)

        self.assertIsNone(self.widget.control)

        with self.event_loop():
            self.widget.create(parent=self.window.control)

        self.assertIsNotNone(self.widget.control)

        with self.event_loop():
            self.widget.destroy()

    def test_horizontal(self):
        # test that horizontal split works
        with self.event_loop():
            self.widget = SplitPanel(
                self.window.control, direction="horizontal"
            )
        with self.event_loop():
            self.widget.destroy()

    def test_ratio(self):
        # test that ratio works
        with self.event_loop():
            self.widget = SplitPanel(self.window.control, ratio=0.25)
        with self.event_loop():
            self.widget.destroy()

    def test_contents(self):
        # test that contents works
        self.widget = SplitPanel(
            self.window.control, lhs=HeadingText, rhs=HeadingText
        )
        with self.event_loop():
            self.widget.create()

        with self.event_loop():
            self.widget.destroy()

    def test_contents_toolkit_control(self):
        # test that toolkit control contents works
        def create_toolkit_control(parent):
            widget = HeadingText(parent)
            widget.create()
            return widget.control

        self.widget = SplitPanel(
            self.window.control,
            lhs=create_toolkit_control,
            rhs=create_toolkit_control,
        )
        with self.event_loop():
            self.widget.create()

        with self.event_loop():
            self.widget.destroy()



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought util package component>

from pyface.timer.timer import CallbackTimer, Timer


class DoLaterTimer(Timer):
    """ Performs a callback once at a later time.

    This is not used by the `do_later` functions and is only provided for
    backwards compatibility of the API.
    """

    #: The perform the callback once.
    repeat = 1

    def __init__(self, interval, callable, args, kw_args):
        # Adapt the old DoLaterTimer initializer to the Timer initializer.
        super().__init__(interval, callable, *args, **kw_args)


def do_later(callable, *args, **kwargs):
    """ Does something 50 milliseconds from now.

    Parameters
    ----------
    callable : Callable
        The callable to call in 50ms time.
    args, kwargs : tuple, dict
        Arguments to be passed through to the callable.
    """
    return CallbackTimer.single_shot(
        interval=0.05, callback=callable, args=args, kwargs=kwargs
    )


def do_after(interval, callable, *args, **kwargs):
    """ Does something after some specified time interval.

    Parameters
    ----------
    interval : float
        The time interval in milliseconds to wait before calling.
    callable : Callable
        The callable to call.
    args
        Positional arguments to be passed through to the callable.
    kwargs
        Keyword arguments to be passed through to the callable.
        Arguments to be passed through to the callable.
    """
    return CallbackTimer.single_shot(
        interval=interval / 1000.0, callback=callable, args=args, kwargs=kwargs
    )



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


from traits.api import Any, Bool, Interface, List, Str


class IStatusBarManager(Interface):
    """ The interface for a status bar manager. """

    # The message displayed in the first field of the status bar.
    message = Str()

    # The messages to be displayed in the status bar fields.
    messages = List(Str)

    # The toolkit-specific control that represents the status bar.
    status_bar = Any()

    # Whether to show a size grip on the status bar.
    size_grip = Bool(False)

    # Whether the status bar is visible.
    visible = Bool(True)

    def create_status_bar(self, parent):
        """ Creates a status bar.

        Parameters
        ----------
        parent : toolkit control
            The toolkit control that owns the status bar.
        """

    def destroy(self):
        """ Destroys the status bar. """



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""

API for the ``pyface.data_view.data_models`` subpackage.

Data Accessors
--------------

- :class:`~.AbstractDataAccessor`
- :class:`~.AttributeDataAccessor`
- :class:`~.ConstantDataAccessor`
- :class:`~.IndexDataAccessor`
- :class:`~.KeyDataAccessor`

Data Models
-----------

- :class:`~.RowTableDataModel`
- :class:`~.ArrayDataModel`. Note that this data model is only available if
  ``numpy`` is available in the environment.

"""
try:
    import numpy  # noqa: F401
except ImportError:
    pass
else:
    del numpy
    from .array_data_model import ArrayDataModel  # noqa: F401

from .data_accessors import (  # noqa: F401
    AbstractDataAccessor, AttributeDataAccessor, ConstantDataAccessor,
    IndexDataAccessor, KeyDataAccessor
)
from .row_table_data_model import RowTableDataModel  # noqa: F401



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The base interface for all pyface widgets. """


from traits.api import Any, Bool, HasTraits, Interface, Instance, Str


class IWidget(Interface):
    """ The base interface for all pyface widgets.

    Pyface widgets delegate to a toolkit specific control.
    """

    #: The toolkit specific control that represents the widget.
    control = Any()

    #: The control's optional parent control.
    parent = Any()

    #: Whether or not the control is visible
    visible = Bool(True)

    #: Whether or not the control is enabled
    enabled = Bool(True)

    #: A tooltip for the widget.
    tooltip = Str()

    #: An optional context menu for the widget.
    context_menu = Instance("pyface.action.menu_manager.MenuManager")

    # ------------------------------------------------------------------------
    # 'IWidget' interface.
    # ------------------------------------------------------------------------

    def show(self, visible):
        """ Show or hide the widget.

        Parameters
        ----------
        visible : bool
            Visible should be ``True`` if the widget should be shown.
        """

    def enable(self, enabled):
        """ Enable or disable the widget.

        Parameters
        ----------
        enabled : bool
            The enabled state to set the widget to.
        """

    def focus(self):
        """ Set the keyboard focus to this widget.
        """

    def has_focus(self):
        """ Does the widget currently have keyboard focus?

        Returns
        -------
        focus_state : bool
            Whether or not the widget has keyboard focus.
        """

    def create(self, parent=None):
        """ Creates the toolkit specific control.

        This method should create the control and assign it to the
        :py:attr:`control` trait.
        """

    def destroy(self):
        """ Destroy the control if it exists. """

    # ------------------------------------------------------------------------
    # Protected 'IWidget' interface.
    # ------------------------------------------------------------------------

    def _create_control(self, parent):
        """ Create toolkit specific control that represents the widget.

        Parameters
        ----------
        parent : toolkit control
            The toolkit control to be used as the parent for the widget's
            control.

        Returns
        -------
        control : toolkit control
            A control for the widget.
        """

    def _add_event_listeners(self):
        """ Set up toolkit-specific bindings for events """

    def _remove_event_listeners(self):
        """ Remove toolkit-specific bindings for events """


class MWidget(HasTraits):
    """ The mixin class that contains common code for toolkit specific
    implementations of the IWidget interface.
    """

    #: A tooltip for the widget.
    tooltip = Str()

    #: An optional context menu for the widget.
    context_menu = Instance("pyface.action.menu_manager.MenuManager")

    def create(self, parent=None):
        """ Creates the toolkit specific control.

        This method should create the control and assign it to the
        :py:attr:``control`` trait.
        """
        if parent is not None:
            self.parent = parent
        self.control = self._create_control(self.parent)
        self._initialize_control()
        self._add_event_listeners()

    def destroy(self):
        """ Call clean-up code and destroy toolkit objects.

        Subclasses should override to perform any additional clean-up, ensuring
        that they call super() after that clean-up.
        """
        if self.control is not None:
            self._remove_event_listeners()
            self.control = None

    # ------------------------------------------------------------------------
    # Protected 'IWidget' interface.
    # ------------------------------------------------------------------------

    def _create(self):
        """ Creates the toolkit specific control.

        The default implementation simply calls create()
        """
        from warnings import warn

        warn(
            "The _create() method will be removed in a future version of "
            "Pyface.  Use create() instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        self.create()

    def _create_control(self, parent):
        """ Create toolkit specific control that represents the widget.

        Parameters
        ----------
        parent : toolkit control
            The toolkit control to be used as the parent for the widget's
            control.

        Returns
        -------
        control : toolkit control
            A control for the widget.
        """
        raise NotImplementedError()

    def _initialize_control(self):
        """ Perform any post-creation initialization for the control.
        """
        self._set_control_tooltip(self.tooltip)

    def _add_event_listeners(self):
        """ Set up toolkit-specific bindings for events """
        self.observe(self._tooltip_updated, "tooltip", dispatch="ui")
        self.observe(
            self._context_menu_updated, "context_menu", dispatch="ui"
        )
        if self.control is not None and self.context_menu is not None:
            self._observe_control_context_menu()

    def _remove_event_listeners(self):
        """ Remove toolkit-specific bindings for events """
        if self.control is not None and self.context_menu is not None:
            self._observe_control_context_menu(remove=True)
        self.observe(
            self._context_menu_updated,
            "context_menu",
            dispatch="ui",
            remove=True,
        )
        self.observe(
            self._tooltip_updated, "tooltip", dispatch="ui", remove=True
        )

    # Toolkit control interface ---------------------------------------------

    def _get_control_tooltip(self):
        """ Toolkit specific method to get the control's tooltip. """
        raise NotImplementedError()

    def _set_control_tooltip(self, tooltip):
        """ Toolkit specific method to set the control's tooltip. """
        raise NotImplementedError()

    def _observe_control_context_menu(self, remove=False):
        """ Toolkit specific method to change the context menu observer.

        This should use _handle_control_context_menu as the event handler.

        Parameters
        ----------
        remove : bool
            Whether the context menu handler should be removed or added.
        """
        raise NotImplementedError()

    def _handle_control_context_menu(self, event):
        """ Handle a context menu event.

        Implementations should override this with a method suitable to be used
        as a toolkit event handler that invokes a context menu.

        The function signature will likely vary from toolkit to toolkit.
        """
        raise NotImplementedError()

    # Trait change handlers -------------------------------------------------

    def _tooltip_updated(self, event):
        tooltip = event.new
        if self.control is not None:
            self._set_control_tooltip(tooltip)

    def _context_menu_updated(self, event):
        if self.control is not None:
            if event.new is None:
                self._observe_control_context_menu(remove=True)
            if event.old is None:
                self._observe_control_context_menu()



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


""" Defines common traits used within the pyface library. """
from collections.abc import Sequence
import logging

try:
    import numpy as np
except ImportError:
    np = None

from traits.api import (
    ABCHasStrictTraits,
    DefaultValue,
    Enum,
    Range,
    TraitError,
    TraitFactory,
    TraitType,
)
from traits.trait_base import get_resource_path

from pyface.color import Color
from pyface.font import Font
from pyface.i_image import IImage
from pyface.util.color_parser import ColorParseError
from pyface.util.font_parser import simple_parser, FontParseError


logger = logging.getLogger(__name__)

# -------------------------------------------------------------------------------
#  Images
# -------------------------------------------------------------------------------

# cache of lookups from string to ImageResource instance
image_resource_cache = {}

# cache of conversions of ImageResource instances to toolkit bitmaps
image_bitmap_cache = {}


def convert_image(value, level=3):
    """ Converts a specified value to an ImageResource if possible.
    """
    if not isinstance(value, str):
        return value

    key = value
    is_pyface_image = value.startswith("@")
    if not is_pyface_image:
        search_path = get_resource_path(level)
        key = "%s[%s]" % (value, search_path)

    result = image_resource_cache.get(key)
    if result is None:
        if is_pyface_image:
            try:
                from .image.image import ImageLibrary

                result = ImageLibrary.image_resource(value)
            except Exception as exc:
                logger.error("Can't load image resource '%s'." % value)
                logger.exception(exc)
                result = None
        else:
            from pyface.image_resource import ImageResource

            result = ImageResource(value, search_path=[search_path])

        image_resource_cache[key] = result

    return result


def convert_bitmap(image):
    """ Converts an ImageResource to a bitmap using a cache.
    """
    from pyface.i_image_resource import IImageResource
    if not isinstance(image, IImageResource):
        # don't try to cache non-ImageResource IImages as they may be
        # dynamically changing
        return image.create_bitmap()

    bitmap = image_bitmap_cache.get(image)
    if (bitmap is None) and (image is not None):
        image_bitmap_cache[image] = bitmap = image.create_bitmap()

    return bitmap


class Image(TraitType):
    """ Defines a trait whose value must be a IImage or a string
        that can be converted to an IImageResource.
    """

    #: Define the default value for the trait.
    default_value = None

    #: A description of the type of value this trait accepts.
    info_text = "an IImage or string that can be used to define an ImageResource"  # noqa: E501

    def __init__(self, value=None, **metadata):
        """ Creates an Image trait.

        Parameters
        ----------
        value : string or ImageResource
            The default value for the Image, either an IImage object,
            or a string from which an ImageResource object can be derived.
        """
        super().__init__(convert_image(value), **metadata)

    def validate(self, object, name, value):
        """ Validates that a specified value is valid for this trait.
        """
        if value is None:
            return None

        new_value = convert_image(value, 4)
        if isinstance(new_value, IImage):
            return new_value

        self.error(object, name, value)

    def create_editor(self):
        """ Returns the default UI editor for the trait.
        """
        from traitsui.editors.api import ImageEditor

        return ImageEditor()


# -------------------------------------------------------------------------------
#  Color
# -------------------------------------------------------------------------------


class PyfaceColor(TraitType):
    """ A Trait which casts strings and tuples to a Pyface Color value.
    """

    #: The default value should be a tuple (factory, args, kwargs).
    default_value_type = DefaultValue.callable_and_args

    def __init__(self, value=None, **metadata):
        if value is not None:
            color = self.validate(None, None, value)
            default_value = (Color, (), {'rgba': color.rgba})
        else:
            default_value = (Color, (), {})
        super().__init__(default_value, **metadata)

    def validate(self, object, name, value):
        """Validate the trait

        This accepts, Color values, parseable strings and RGB(A) sequences
        (including numpy arrays).
        """
        if isinstance(value, Color):
            return value
        if isinstance(value, str):
            try:
                return Color.from_str(value)
            except ColorParseError:
                self.error(object, name, value)
        is_array = (
            np is not None
            and isinstance(value, (np.ndarray, np.void))
        )
        if is_array or isinstance(value, Sequence):
            channels = tuple(value)
            if len(channels) == 4:
                return Color(rgba=channels)
            elif len(channels) == 3:
                return Color(rgb=channels)

        self.error(object, name, value)

    def info(self):
        """Describe the trait"""
        return (
            "a Pyface Color, a #-hexadecimal rgb or rgba string,  a standard "
            "color name, or a sequence of RGBA or RGB values between 0 and 1"
        )


# -------------------------------------------------------------------------------
#  Font
# -------------------------------------------------------------------------------


class PyfaceFont(TraitType):
    """ A Trait which casts strings to a Pyface Font value.
    """

    #: The default value should be a tuple (factory, args, kwargs)
    default_value_type = DefaultValue.callable_and_args

    #: The parser to use when converting text to keyword args.  This should
    #: accept a string and return a dictionary of Font class trait values (ie.
    #: "family", "size", "weight", etc.).
    parser = None

    def __init__(self, value=None, *, parser=simple_parser, **metadata):
        self.parser = parser
        if value is not None:
            try:
                font = self.validate(None, None, value)
            except TraitError:
                raise ValueError(
                    "expected " + self.info()
                    + f", but got {value!r}"
                )
            default_value = (
                Font,
                (),
                font.trait_get(transient=lambda x: not x),
            )
        else:
            default_value = (Font, (), {})
        super().__init__(default_value, **metadata)

    def validate(self, object, name, value):
        """Validate the trait

        This accepts, Font values and parseable strings.
        """
        if isinstance(value, Font):
            return value
        if isinstance(value, str):
            try:
                return Font(**self.parser(value))
            except FontParseError:
                self.error(object, name, value)

        self.error(object, name, value)

    def info(self):
        """Describe the trait"""
        return (
            "a Pyface Font, or a string describing a Pyface Font"
        )


# -------------------------------------------------------------------------------
#  Borders, Margins and Layout
# -------------------------------------------------------------------------------


class BaseMB(ABCHasStrictTraits):
    """ Base class for Margins and Borders

    The constructor of this class maps posiitonal arguments to traits.

    - If one value is provided it is taken as the value for all sides.
    - If two values are provided, then the first argument is used for
      left and right, while the second is used for top and bottom.
    - If 4 values are provided, then the arguments are mapped to
      left, right, top, and bottom, respectively.
    """

    def __init__(self, *args, **traits):
        n = len(args)
        if n > 0:
            if n == 1:
                left = right = top = bottom = args[0]
            elif n == 2:
                left = right = args[0]
                top = bottom = args[1]
            elif n == 4:
                left, right, top, bottom = args
            else:
                raise TraitError(
                    "0, 1, 2 or 4 arguments expected, but %d " "specified" % n
                )
            traits.update(
                {"left": left, "right": right, "top": top, "bottom": bottom}
            )

        super().__init__(**traits)


class Margin(BaseMB):
    """A HasTraits class that holds margin sizes."""

    #: The amount of padding/margin at the top.
    top = Range(-32, 32, 0)

    #: The amount of padding/margin at the bottom.
    bottom = Range(-32, 32, 0)

    #: The amount of padding/margin on the left.
    left = Range(-32, 32, 0)

    #: The amount of padding/margin on the right.
    right = Range(-32, 32, 0)


class Border(BaseMB):
    """A HasTraits class that holds border thicknesses."""

    #: The amount of border at the top.
    top = Range(0, 32, 0)

    #: The amount of border at the bottom.
    bottom = Range(0, 32, 0)

    #: The amount of border on the left.
    left = Range(0, 32, 0)

    #: The amount of border on the right.
    right = Range(0, 32, 0)


class HasMargin(TraitType):
    """ Defines a trait whose value must be a Margin object or an integer or
        tuple value that can be converted to one.
    """

    #: The desired value class.
    klass = Margin

    #: Define the default value for the trait.
    default_value = Margin(0)

    #: A description of the type of value this trait accepts.
    info_text = (
        "a Margin instance, or an integer in the range from -32 to 32 "
        "or a tuple with 1, 2 or 4 integers in that range that can be "
        "used to define one"
    )

    def validate(self, object, name, value):
        """ Validates that a specified value is valid for this trait.
        """
        if isinstance(value, int):
            try:
                value = self.klass(value)
            except Exception:
                self.error(object, name, value)
        elif isinstance(value, tuple):
            try:
                value = self.klass(*value)
            except Exception:
                self.error(object, name, value)

        if isinstance(value, self.klass):
            return value

        self.error(object, name, value)

    def get_default_value(self):
        """ Returns a tuple of the form:
                (default_value_type, default_value)

            which describes the default value for this trait.
        """
        dv = self.default_value
        dvt = self.default_value_type
        if dvt < 0:
            if isinstance(dv, int):
                dv = self.klass(dv)
            elif isinstance(dv, tuple):
                dv = self.klass(*dv)

            if not isinstance(dv, self.klass):
                return super().get_default_value()

            self.default_value_type = dvt = DefaultValue.callable_and_args
            dv = (self.klass, (), dv.trait_get())

        return (dvt, dv)


class HasBorder(HasMargin):
    """ Defines a trait whose value must be a Border object or an integer
        or tuple value that can be converted to one.
    """

    #: The desired value class.
    klass = Border

    #: Define the default value for the trait.
    default_value = Border(0)

    #: A description of the type of value this trait accepts.
    info_text = (
        "a Border instance, or an integer in the range from 0 to 32 "
        "or a tuple with 1, 2 or 4 integers in that range that can be "
        "used to define one"
    )


#: The position of an image relative to its associated text.
Position = Enum("left", "right", "above", "below")

#: The alignment of text within a control.
Alignment = Enum("default", "left", "center", "right")

#: Whether the orientation of a widget's contents is horizontal or vertical.
Orientation = Enum("vertical", "horizontal")


# -------------------------------------------------------------------------------
#  Legacy TraitsUI Color and Font Traits
# -------------------------------------------------------------------------------

def TraitsUIColor(*args, **metadata):
    """ Returns a trait whose value must be a GUI toolkit-specific color.

    This is copied from the deprecated trait that is in traits.api.  It adds
    a deferred dependency on TraitsUI.

    This trait will be replaced by native Pyface color traits in Pyface 8.0.
    New code should not use this trait.
    """
    from traitsui.toolkit_traits import ColorTrait

    return ColorTrait(*args, **metadata)


TraitsUIColor = TraitFactory(TraitsUIColor)


def TraitsUIFont(*args, **metadata):
    """ Returns a trait whose value must be a GUI toolkit-specific font.

    This is copied from the deprecated trait that is in traits.api.  It adds
    a deferred dependency on TraitsUI.

    This trait will be replaced by native Pyface font traits in Pyface 8.0.
    New code should not use this trait.
    """
    from traitsui.toolkit_traits import FontTrait

    return FontTrait(*args, **metadata)


TraitsUIFont = TraitFactory(TraitsUIFont)



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" The interface for a PIL Image. """

from traits.api import HasStrictTraits, Instance

from pyface.i_image import IImage


class IPILImage(IImage):
    """ The interface for a image that wraps a PIL Image.
    """

    # 'IPILImage' interface --------------------------------------------

    #: The PIL Image instance.
    image = Instance("PIL.Image.Image")


class MPILImage(HasStrictTraits):
    """ The base implementation mixin for a image that wraps a PIL Image.
    """

    # 'IPILImage' interface --------------------------------------------

    #: The PIL Image instance.
    image = Instance("PIL.Image.Image")

    def __init__(self, image, **traits):
        super().__init__(image=image, **traits)

    def create_bitmap(self, size=None):
        """ Creates a bitmap image for this image.

        Parameters
        ----------
        size : (int, int) or None
            The desired size as a (width, height) tuple, or None if wanting
            default image size.

        Returns
        -------
        image : bitmap
            The toolkit bitmap corresponding to the image and the specified
            size.
        """
        from pyface.util.image_helpers import image_to_bitmap
        return image_to_bitmap(self.create_image(size))

    def create_icon(self, size=None):
        """ Creates an icon for this image.

        Parameters
        ----------
        size : (int, int) or None
            The desired size as a (width, height) tuple, or None if wanting
            default icon size.

        Returns
        -------
        image : icon
            The toolkit image corresponding to the image and the specified
            size as an icon.
        """
        from pyface.util.image_helpers import bitmap_to_icon
        return bitmap_to_icon(self.create_bitmap(size))



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The Wx-specific implementation of the label field class """

import wx

from traits.api import provides

from pyface.fields.i_label_field import ILabelField, MLabelField
from .field import Field


@provides(ILabelField)
class LabelField(MLabelField, Field):
    """ The Wx-specific implementation of the label field class """

    def _create_control(self, parent):
        """ Create the toolkit-specific control that represents the widget. """
        control = wx.StaticText(parent)
        return control

    # ------------------------------------------------------------------------
    # Private interface
    # ------------------------------------------------------------------------

    # Toolkit control interface ---------------------------------------------

    def _get_control_value(self):
        """ Toolkit specific method to get the control's value. """
        return self.control.GetLabel()

    def _set_control_value(self, value):
        """ Toolkit specific method to set the control's value. """
        self.control.SetLabel(value)

    def _set_control_icon(self, icon):
        """ Toolkit specific method to set the control's icon. """
        # don't support icons on Wx for now
        pass



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import unittest


from pyface.image_resource import ImageResource
from ..toggle_field import (
    CheckBoxField, RadioButtonField, ToggleButtonField
)
from .field_mixin import FieldMixin


class ToggleFieldMixin(FieldMixin):

    # Tests ------------------------------------------------------------------

    def test_toggle_field(self):
        self._create_widget_control()

        self.widget.value = True
        self.gui.process_events()

        self.assertEqual(self.widget._get_control_value(), True)

    def test_toggle_field_set(self):
        self._create_widget_control()

        with self.assertTraitChanges(self.widget, "value", count=1):
            self.widget._set_control_value(True)
            self.gui.process_events()

        self.assertEqual(self.widget.value, True)

    def test_text_field_text(self):
        self._create_widget_control()
        self.assertEqual(self.widget._get_control_text(), "Toggle")

        self.widget.text = "Test"
        self.gui.process_events()

        self.assertEqual(self.widget._get_control_text(), "Test")

    def test_text_field_image(self):
        self._create_widget_control()
        image = ImageResource("question")

        # XXX can't validate icon values currently, so just a smoke test
        self.widget.image = image
        self.gui.process_events()


class TestCheckboxField(ToggleFieldMixin, unittest.TestCase):

    def _create_widget_simple(self, **traits):
        traits.setdefault("text", "Toggle")
        traits.setdefault("tooltip", "Dummy")
        return CheckBoxField(**traits)


class TestRadioButtonField(ToggleFieldMixin, unittest.TestCase):

    def _create_widget_simple(self, **traits):
        traits.setdefault("text", "Toggle")
        traits.setdefault("tooltip", "Dummy")
        return RadioButtonField(**traits)


class TestToggleButtonField(ToggleFieldMixin, unittest.TestCase):

    def _create_widget_simple(self, **traits):
        traits.setdefault("text", "Toggle")
        traits.setdefault("tooltip", "Dummy")
        return ToggleButtonField(**traits)



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Qt alignment helper functions """

from pyface.qt.QtCore import Qt


ALIGNMENT_TO_QALIGNMENT = {
    "default": Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter,
    "left": Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter,
    "center": Qt.AlignmentFlag.AlignHCenter | Qt.AlignmentFlag.AlignVCenter,
    "right": Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter,
}
QALIGNMENT_TO_ALIGNMENT = {
    0: "default",
    int(Qt.AlignmentFlag.AlignLeft): "left",
    int(Qt.AlignmentFlag.AlignHCenter): "center",
    int(Qt.AlignmentFlag.AlignRight): "right",
}
ALIGNMENT_MASK = (
    Qt.AlignmentFlag.AlignLeft
    | Qt.AlignmentFlag.AlignHCenter
    | Qt.AlignmentFlag.AlignRight
)


def alignment_to_qalignment(alignment):
    """Convert an Alignment trait value to a Qt Alignment

    Parameters
    ----------
    alignment : str
        An Alignment trait value, one of "default", "left", "center", or
        "right".

    Returns
    -------
    qalignment : Qt.AlignmentFlag
        A Qt.AlignmentFlag value
    """
    return ALIGNMENT_TO_QALIGNMENT[alignment]


def qalignment_to_alignment(alignment):
    """Convert a Qt Alignment value to an Alignment trait

    Parameters
    ----------
    qalignment : Qt.AlignmentFlag
        A Qt.AlignmentFlag value

    Returns
    -------
    alignment : str
        An Alignment trait value, one of "default", "left", "center", or
        "right".
    """
    h_alignment = int(alignment & ALIGNMENT_MASK)
    return QALIGNMENT_TO_ALIGNMENT[h_alignment]



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traits.api import Callable, List

from .editable_value import EditableValue


class EnumValue(EditableValue):
    """ Editable value that takes one of a collection of pre-set values.

    Each value can be associated with text, colors and images by supplying
    functions ``format``, ``colors`` and ``images``, respectively.
    """

    #: The list of values which are allowed for the value.
    values = List()

    #: A function that converts a value to a string for display.
    format = Callable(str, update_value_type=True)

    #: A map from valid values to colors.
    colors = Callable(None, update_value_type=True)

    #: A map from valid values to images.
    images = Callable(None, update_value_type=True)

    def is_valid(self, model, row, column, value):
        """ Whether or not the value is valid for the data item specified.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.
        value : Any
            The value to validate.

        Returns
        -------
        is_valid : bool
            Whether or not the value is valid.
        """
        return value in self.values

    def has_text(self, model, row, column):
        """ Get the display text from the underlying value.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        text : str
            The text to display.
        """
        return self.format is not None

    def get_text(self, model, row, column):
        """ Get the display text from the underlying value.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        text : str
            The text to display.
        """
        return self.format(model.get_value(row, column))

    def has_color(self, model, row, column):
        """ Whether or not the value has color data.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        has_color : bool
            Whether or not the value has data-associated color
            values.
        """
        return self.colors is not None

    def get_color(self, model, row, column):
        """ Get data-associated colour values for the given item.

        The default implementation returns white.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        color : Color
            The color associated with the cell.
        """
        return self.colors(model.get_value(row, column))

    def has_image(self, model, row, column):
        """ Whether or not the value has an image associated with it.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        has_image : bool
            Whether or not the value has an image associated with it.
        """
        return self.images is not None

    def get_image(self, model, row, column):
        """ An image associated with the underlying value.

        The default implementation returns None.

        Parameters
        ----------
        model : AbstractDataModel
            The data model holding the data.
        row : sequence of int
            The row in the data model being queried.
        column : sequence of int
            The column in the data model being queried.

        Returns
        -------
        image : IImage
            The image associated with the underlying value.
        """
        return self.images(model.get_value(row, column))



# Copyright (c) 2005-2023, Enthought Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!

import unittest

import wx

from traits.testing.optional_dependencies import numpy as np, requires_numpy
from ..image_helpers import (
    bitmap_to_icon, bitmap_to_image, image_to_array, image_to_bitmap,
    array_to_image, AspectRatio, ScaleMode, resize_image, resize_bitmap,
)


class TestImageHelpers(unittest.TestCase):

    def test_image_to_bitmap(self):
        wximage = wx.Image(32, 64)
        wximage.SetRGB(wx.Rect(0, 0, 32, 64), 0x44, 0x88, 0xcc)

        wxbitmap = image_to_bitmap(wximage)

        self.assertIsInstance(wxbitmap, wx.Bitmap)

    def test_bitmap_to_image(self):
        wximage = wx.Image(32, 64)
        wximage.SetRGB(wx.Rect(0, 0, 32, 64), 0x44, 0x88, 0xcc)
        wxbitmap = wximage.ConvertToBitmap()

        wximage = bitmap_to_image(wxbitmap)

        self.assertIsInstance(wximage, wx.Image)

    def test_bitmap_to_icon(self):
        wximage = wx.Image(32, 64)
        wximage.SetRGB(wx.Rect(0, 0, 32, 64), 0x44, 0x88, 0xcc)
        wxbitmap = wximage.ConvertToBitmap()

        wximage = bitmap_to_icon(wxbitmap)

        self.assertIsInstance(wximage, wx.Icon)

    def test_resize_image(self):
        wximage = wx.Image(32, 64)
        wximage.SetRGB(wx.Rect(0, 0, 32, 64), 0x44, 0x88, 0xcc)

        wximage = resize_image(wximage, (128, 128))

        self.assertIsInstance(wximage, wx.Image)
        self.assertEqual(wximage.GetWidth(), 128)
        self.assertEqual(wximage.GetHeight(), 128)

    def test_resize_image_smooth(self):
        wximage = wx.Image(32, 64)
        wximage.SetRGB(wx.Rect(0, 0, 32, 64), 0x44, 0x88, 0xcc)

        wximage = resize_image(wximage, (128, 128), mode=ScaleMode.smooth)

        self.assertIsInstance(wximage, wx.Image)
        self.assertEqual(wximage.GetWidth(), 128)
        self.assertEqual(wximage.GetHeight(), 128)

    def test_resize_image_constrain(self):
        wximage = wx.Image(32, 64)
        wximage.SetRGB(wx.Rect(0, 0, 32, 64), 0x44, 0x88, 0xcc)

        wximage = resize_image(wximage, (128, 128), AspectRatio.keep_constrain)

        self.assertIsInstance(wximage, wx.Image)
        self.assertEqual(wximage.GetWidth(), 64)
        self.assertEqual(wximage.GetHeight(), 128)

    def test_resize_image_expand(self):
        wximage = wx.Image(32, 64)
        wximage.SetRGB(wx.Rect(0, 0, 32, 64), 0x44, 0x88, 0xcc)

        wximage = resize_image(wximage, (128, 128), AspectRatio.keep_expand)

        self.assertIsInstance(wximage, wx.Image)
        self.assertEqual(wximage.GetWidth(), 128)
        self.assertEqual(wximage.GetHeight(), 256)

    def test_resize_bitmap(self):
        wximage = wx.Image(32, 64)
        wximage.SetRGB(wx.Rect(0, 0, 32, 64), 0x44, 0x88, 0xcc)
        wxbitmap = wximage.ConvertToBitmap()

        wxbitmap = resize_bitmap(wxbitmap, (128, 128))

        self.assertIsInstance(wxbitmap, wx.Bitmap)
        self.assertEqual(wxbitmap.GetWidth(), 128)
        self.assertEqual(wxbitmap.GetHeight(), 128)

    def test_resize_bitmap_smooth(self):
        wximage = wx.Image(32, 64)
        wximage.SetRGB(wx.Rect(0, 0, 32, 64), 0x44, 0x88, 0xcc)
        wxbitmap = wximage.ConvertToBitmap()

        wxbitmap = resize_bitmap(wxbitmap, (128, 128), mode=ScaleMode.smooth)

        self.assertIsInstance(wxbitmap, wx.Bitmap)
        self.assertEqual(wxbitmap.GetWidth(), 128)
        self.assertEqual(wxbitmap.GetHeight(), 128)

    def test_resize_bitmap_constrain(self):
        wximage = wx.Image(32, 64)
        wximage.SetRGB(wx.Rect(0, 0, 32, 64), 0x44, 0x88, 0xcc)
        wxbitmap = wximage.ConvertToBitmap()

        wxbitmap = resize_bitmap(wxbitmap, (128, 128), AspectRatio.keep_constrain)

        self.assertIsInstance(wxbitmap, wx.Bitmap)
        self.assertEqual(wxbitmap.GetWidth(), 64)
        self.assertEqual(wxbitmap.GetHeight(), 128)

    def test_resize_bitmap_expand(self):
        wximage = wx.Image(32, 64)
        wximage.SetRGB(wx.Rect(0, 0, 32, 64), 0x44, 0x88, 0xcc)
        wxbitmap = wximage.ConvertToBitmap()

        wxbitmap = resize_bitmap(wxbitmap, (128, 128), AspectRatio.keep_expand)

        self.assertIsInstance(wxbitmap, wx.Bitmap)
        self.assertEqual(wxbitmap.GetWidth(), 128)
        self.assertEqual(wxbitmap.GetHeight(), 256)


@requires_numpy
class TestArrayImageHelpers(unittest.TestCase):

    def test_image_to_array_rgb(self):
        wximage = wx.Image(32, 64)
        wximage.SetRGB(wx.Rect(0, 0, 32, 64), 0x44, 0x88, 0xcc)

        array = image_to_array(wximage)

        self.assertEqual(array.shape, (32, 64, 4))
        self.assertEqual(array.dtype, np.dtype('uint8'))
        self.assertTrue(np.all(array[:, :, 3] == 0xff))
        self.assertTrue(np.all(array[:, :, 0] == 0x44))
        self.assertTrue(np.all(array[:, :, 1] == 0x88))
        self.assertTrue(np.all(array[:, :, 2] == 0xcc))

    def test_image_to_array_rgba(self):
        wximage = wx.Image(32, 64)
        wximage.SetRGB(wx.Rect(0, 0, 32, 64), 0x44, 0x88, 0xcc)
        wximage.InitAlpha()
        wximage.SetAlpha(np.full((32*64,), 0xee, dtype='uint8'))

        array = image_to_array(wximage)

        self.assertEqual(array.shape, (32, 64, 4))
        self.assertEqual(array.dtype, np.dtype('uint8'))
        self.assertTrue(np.all(array[:, :, 0] == 0x44))
        self.assertTrue(np.all(array[:, :, 1] == 0x88))
        self.assertTrue(np.all(array[:, :, 2] == 0xcc))
        self.assertTrue(np.all(array[:, :, 3] == 0xee))

    def test_array_to_image_rgb(self):
        array = np.empty((64, 32, 3), dtype='uint8')
        array[:, :, 0] = 0x44
        array[:, :, 1] = 0x88
        array[:, :, 2] = 0xcc

        wximage = array_to_image(array)

        self.assertEqual(wximage.GetWidth(), 32)
        self.assertEqual(wximage.GetHeight(), 64)
        self.assertFalse(wximage.HasAlpha())

    def test_array_to_image_rgba(self):
        array = np.empty((64, 32, 4), dtype='uint8')
        array[:, :, 0] = 0x44
        array[:, :, 1] = 0x88
        array[:, :, 2] = 0xcc
        array[:, :, 3] = 0xee

        wximage = array_to_image(array)

        self.assertEqual(wximage.GetWidth(), 32)
        self.assertEqual(wximage.GetHeight(), 64)
        self.assertTrue(wximage.HasAlpha())

    def test_array_to_image_bad_channels(self):
        array = np.empty((64, 32, 2), dtype='uint8')
        array[:, :, 0] = 0x44
        array[:, :, 1] = 0x88

        with self.assertRaises(ValueError):
            array_to_image(array)

    def test_array_to_image_bad_ndim(self):
        array = np.full((64, 32), 0x44, dtype='uint8')

        with self.assertRaises(ValueError):
            array_to_image(array)



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The Qt-specific implementation of the toggle field class """


from traits.api import provides

from pyface.fields.i_toggle_field import IToggleField, MToggleField
from pyface.qt.QtGui import (
    QCheckBox, QIcon, QPushButton, QRadioButton
)
from .editable_field import EditableField


@provides(IToggleField)
class ToggleField(MToggleField, EditableField):
    """ The Qt-specific implementation of the toggle field class """

    # ------------------------------------------------------------------------
    # Private interface
    # ------------------------------------------------------------------------

    # Toolkit control interface ---------------------------------------------

    def _get_control_value(self):
        """ Toolkit specific method to get the control's value. """
        return self.control.isChecked()

    def _get_control_text(self):
        """ Toolkit specific method to get the control's text. """
        return self.control.text()

    def _set_control_value(self, value):
        """ Toolkit specific method to set the control's value. """
        return self.control.setChecked(value)

    def _set_control_text(self, text):
        """ Toolkit specific method to set the control's text. """
        return self.control.setText(text)

    def _set_control_icon(self, icon):
        """ Toolkit specific method to set the control's icon. """
        if icon is not None:
            self.control.setIcon(icon.create_icon())
        else:
            self.control.setIcon(QIcon())

    def _observe_control_value(self, remove=False):
        """ Toolkit specific method to change the control value observer. """
        if remove:
            self.control.toggled.disconnect(self._update_value)
        else:
            self.control.toggled.connect(self._update_value)

    def _get_control_alignment(self):
        """ Toolkit specific method to get the control's alignment. """
        # dummy implementation
        return self.alignment

    def _set_control_alignment(self, alignment):
        """ Toolkit specific method to set the control's alignment. """
        # use stylesheet for button alignment
        self.control.setStyleSheet(f"text-align: {alignment}")


class CheckBoxField(ToggleField):
    """ The Qt-specific implementation of the checkbox class """

    def _create_control(self, parent):
        """ Create the toolkit-specific control that represents the widget. """
        control = QCheckBox(parent)
        return control


class RadioButtonField(ToggleField):
    """ The Qt-specific implementation of the radio button class

    This is intended to be used in groups, and shouldn't be used by itself.
    """

    def _create_control(self, parent):
        """ Create the toolkit-specific control that represents the widget. """
        control = QRadioButton(parent)
        return control


class ToggleButtonField(ToggleField):
    """ The Qt-specific implementation of the toggle button class """

    def _create_control(self, parent):
        """ Create the toolkit-specific control that represents the widget. """
        control = QPushButton(parent)
        control.setCheckable(True)
        return control



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from unittest import TestCase
from unittest.mock import Mock

from pyface.data_view.data_view_errors import DataViewSetError
from pyface.data_view.value_types.enum_value import EnumValue


class TestEnumValue(TestCase):

    def setUp(self):
        self.model = Mock()
        self.model.get_value = Mock(return_value=1)
        self.model.can_set_value = Mock(return_value=True)
        self.model.set_value = Mock()

    def test_defaults(self):
        value = EnumValue()
        self.assertTrue(value.is_editable)

    def test_is_valid(self):
        value = EnumValue(values=[1, 2])
        self.assertTrue(value.is_valid(None, [0], [0], 1))

    def test_is_valid_false(self):
        value = EnumValue(values=[2])
        self.assertFalse(value.is_valid(None, [0], [0], 1))

    def test_get_editor_value(self):
        value = EnumValue()
        editable = value.get_editor_value(self.model, [0], [0])
        self.assertEqual(editable, 1)

    def test_set_editor_value(self):
        value = EnumValue(values=[1, 2])
        value.set_editor_value(self.model, [0], [0], 2)
        self.model.set_value.assert_called_once_with([0], [0], 2)

    def test_set_editor_value_bad(self):
        value = EnumValue(values=[1])
        with self.assertRaises(DataViewSetError):
            value.set_editor_value(self.model, [0], [0], 2)

    def test_has_text(self):
        value = EnumValue(values=[1, 2])
        has_text = value.has_text(self.model, [0], [0])
        self.assertTrue(has_text)

    def test_has_text_false(self):
        value = EnumValue(values=[1, 2], format=None)
        has_text = value.has_text(self.model, [0], [0])
        self.assertFalse(has_text)

    def test_get_text(self):
        value = EnumValue(values=[1, 2])
        text = value.get_text(self.model, [0], [0])
        self.assertEqual(text, "1")

    def test_has_color_false(self):
        value = EnumValue(values=[1, 2])
        has_color = value.has_color(self.model, [0], [0])
        self.assertFalse(has_color)

    def test_has_color_true(self):
        value = EnumValue(values=[1, 2], colors=lambda x: "dummy")
        has_color = value.has_color(self.model, [0], [0])
        self.assertTrue(has_color)

    def test_get_color(self):
        value = EnumValue(values=[1, 2], colors=lambda x: "dummy")
        color = value.get_color(self.model, [0], [0])
        self.assertEqual(color, "dummy")

    def test_color_function_none(self):
        value = EnumValue(values=[1, 2], colors=lambda x: None)
        has_color = value.has_color(self.model, [0], [0])
        self.assertTrue(has_color)
        color = value.get_color(self.model, [0], [0])
        self.assertIsNone(color)

    def test_has_image_false(self):
        value = EnumValue(values=[1, 2])
        has_image = value.has_image(self.model, [0], [0])
        self.assertFalse(has_image)

    def test_has_image_true(self):
        value = EnumValue(values=[1, 2], images=lambda x: "dummy")
        has_image = value.has_image(self.model, [0], [0])
        self.assertTrue(has_image)

    def test_get_image(self):
        value = EnumValue(values=[1, 2], images=lambda x: "dummy")
        image = value.get_image(self.model, [0], [0])
        self.assertEqual(image, "dummy")

    def test_image_function_none(self):
        value = EnumValue(values=[1, 2], images=lambda x: None)
        has_image = value.has_image(self.model, [0], [0])
        self.assertTrue(has_image)
        image = value.get_image(self.model, [0], [0])
        self.assertIsNone(image)



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The Wx-specific implementation of the time field class """

from datetime import time

import wx.adv

from traits.api import provides

from pyface.fields.i_time_field import ITimeField, MTimeField
from .editable_field import EditableField


@provides(ITimeField)
class TimeField(MTimeField, EditableField):
    """ The Wx-specific implementation of the time field class """

    # ------------------------------------------------------------------------
    # IWidget interface
    # ------------------------------------------------------------------------

    def _create_control(self, parent):
        """ Create the toolkit-specific control that represents the widget. """

        control = wx.adv.TimePickerCtrl(parent)
        return control

    # ------------------------------------------------------------------------
    # Private interface
    # ------------------------------------------------------------------------

    def _get_control_value(self):
        """ Toolkit specific method to get the control's value. """
        return time(*self.control.GetTime())

    def _set_control_value(self, value):
        """ Toolkit specific method to set the control's value. """
        self.control.SetTime(value.hour, value.minute, value.second)
        wxdatetime = wx.DateTime.Now()
        wxdatetime.SetHour(value.hour)
        wxdatetime.SetMinute(value.minute)
        wxdatetime.SetSecond(value.second)
        event = wx.adv.DateEvent(
            self.control,
            wxdatetime,
            wx.adv.EVT_TIME_CHANGED.typeId
        )
        wx.PostEvent(self.control.GetEventHandler(), event)

    def _observe_control_value(self, remove=False):
        """ Toolkit specific method to change the control value observer. """
        if remove:
            self.control.Unbind(
                wx.adv.EVT_TIME_CHANGED,
                handler=self._update_value
            )
        else:
            self.control.Bind(wx.adv.EVT_TIME_CHANGED, self._update_value)
